/*
 *      mainwindow.c
 *      
 *      Copyright 2009 - 2011 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <glib/gi18n.h>
#include <signal.h>
#include <stdint.h>
#include <stdlib.h>
#include <math.h>

#include "mainwindow.h"
#include "settings.h"
#include "dvbname.h"
#include "dvbstatus.h"
#include "communication.h"
#include "udpclient.h"
#include "dbstorage.h"
#include "netwindow.h"
#include "chlist.h"
#include "streamconfig.h"
#include "sound.h"
#include "dialogs.h"
#include "grstatus.h"
#include "prefwindow.h"
#include "devwindow.h"
#include "calcwindow.h"
#include "scanwindow.h"
#include "satscanwindow.h"
#include "streamwindow.h"
#include "strcliwindow.h"
#include "tswindow.h"
#include "fswindow.h"
#include "resources.h"


#define THREAD_TIMEOUT  1000000

#define GSF_GETTEXT_PACKAGE "gsf"

struct sstatuswindow {
	//main window
	gboolean visible;
	GtkWidget *win;
	gchar *title;
	GtkWidget *caption;
	GtkWidget *quit, *pref, *tp, *sound;
	GtkWidget *snr, *agc, *ber;
	GtkStatusbar *statusbar;
	GtkGrstatus *grstatus;
	GtkWidget *statussep;
	
	GtkWidget *menubar, *actionsmenu, *actions_menu, *search_menu, *network_menu, *scan_menu, *satscan_menu, *sep_menu, *quit_menu, *editmenu,
	          *edit_menu, *preferences_menu, *device_menu, *sound_menu, *viewmenu, *view_menu, *fullscreen_menu, *streamingmenu, *streaming_menu,
	          *serverconfig_menu, *serverclients_menu, *tsstruct_menu, *record_menu, *udpbc_menu, *udpmc_menu, *tcpuc_menu, *helpmenu, *help_menu,
	          *calc_menu, *about_menu;
	GtkAccelGroup *accel_group;
	GtkToolbar *toolbar;
	GtkToolItem *search_tb, *device_tb, *sound_tb, *fullscreen_tb, *preferences_tb, *exit_tb, *separator1_tb, *separator2_tb;
	GtkWidget *historymenu_tb;
	gboolean sound_lock;
	gboolean streaming_lock;
	GtkStatusIcon *trayicon;
	GtkWidget *traymenu, *servicesmenu;
	GtkWidget *search_tm, *services_tm, *sep_tm, *quit_tm, *entry_sm;
};
struct sstatuswindow statuswindow;

enum {
	SIGNAL_THREAD_POLL = 0,
	SIGNAL_THREAD_FRONTEND_READ,
	SIGNAL_THREAD_FRONTEND_TUNE,
	SIGNAL_THREAD_SATSCAN,
	SIGNAL_THREAD_NET_INIT,
	SIGNAL_THREAD_NET_READ,
	SIGNAL_THREAD_NET_TUNE,
	SIGNAL_THREAD_NET_REFRESH
};

enum {
	SOUND_THREAD_POLL = 0,
	SOUND_THREAD_PLAY
};

enum {
	STATUS_MESSAGE_SVC_NETSARE = 0,
	STATUS_MESSAGE_SVC,
	STATUS_MESSAGE_FR_OPENING,
	STATUS_MESSAGE_FR_OPEN_FAIL,
	STATUS_MESSAGE_TP_SELECTION,
	STATUS_MESSAGE_FR_READONLY_MODE,
	STATUS_MESSAGE_FR_NOT_FOUND,
	STATUS_MESSAGE_TP_TUNING,
	STATUS_MESSAGE_TP_TUNUNG_FAIL,
	STATUS_MESSAGE_DEV_NOT_FOUND
};

struct _signalthread {
	GThread *thread;
	gchar action;
	gpointer data;
	gboolean alive;
} signalthread;

struct _soundthread {
	GThread *thread;
	gchar action;
	gpointer data;
	gboolean alive;
} soundthread;

struct _netstatusthread {
	GThread *thread;
	gchar action;
	gpointer data;
	gboolean alive;
} netstatusthread;

struct _historyentry {
	guint32 id;
	guchar type;
	gchar name[256];
};

GArray *history = NULL;
gchar historylength = 0;

GSList *devices = NULL;

struct _frontend frontend;
struct _service *svc;
struct _demuxer demuxer;
struct _netgate netgate;
struct _streamconfig streamconfig;
struct _globalsettings globalsettings;

static void mainwindow_tune_problem_message_response(GtkDialog *dialog, gint response_id, gpointer data);
static gboolean mainwindow_tune_problem_message(gpointer data);
static void mainwindow_streamserver_restart_response(GtkDialog *dialog, gint response_id, gpointer data);
static gboolean mainwindow_streamserver_restart(gpointer data);
static void mainwindow_stop_signal_handler(int sig, siginfo_t *si, void *unused);
static void mainwindow_service_signal_handler(int sig, siginfo_t *si, void *unused);
static gboolean stop_program(GtkWidget *widget, GdkEvent *event, gpointer data);
static gboolean close_main_window(GtkWidget *widget, GdkEvent  *event, gpointer data);
static gboolean search_open(GtkWidget *widget, GdkEvent *event, gpointer data);
static gboolean devconfig_open(GtkWidget *widget, GdkEvent *event, gpointer data);
static gboolean preferences_open(GtkWidget *widget, GdkEvent *event, gpointer data);
static gboolean network_open(GtkWidget *widget, GdkEvent *event, gpointer data);
static gboolean scan_open(GtkWidget *widget, GdkEvent *event, gpointer data);
static gboolean sconf_open(GtkWidget *widget, GdkEvent *event, gpointer data);
static gboolean tsstruct_open(GtkWidget *widget, GdkEvent *event, gpointer data);
static gboolean calculations_open(GtkWidget *widget, GdkEvent *event, gpointer data);
static gboolean mainwindow_fullscreen_open(GtkWidget *widget, GdkEvent *event, gpointer data);
static gboolean toggle_sound_toolbar(GtkToggleToolButton *toggle_tool_button, gpointer data);
static gboolean toggle_sound_menu(GtkCheckMenuItem *checkmenuitem, gpointer data);
static gboolean toggle_streaming_menu(GtkCheckMenuItem *checkmenuitem, gpointer data);
gpointer sounds_thread(gpointer data);
gpointer signal_thread(gpointer data);
static gint read_event(gint status, uint32_t freq, uint32_t sr, gint fec);
static gboolean read_signal(gpointer data);
gboolean zero_signal(gpointer data);
//static gboolean open_frontend(gpointer data);
static gboolean mainwindow_device_open(struct _device *device);
static gboolean mainwindow_device_close(gpointer data);
static gboolean tune_frontend(gpointer data);
static gboolean mainwindow_status_message(gpointer data);
static gboolean lock_sound_buttons(gboolean lock);
gboolean lock_search_buttons(gpointer data);
static gboolean lock_scan_buttons(gboolean lock);
void statuswindow_trayaction(GtkStatusIcon *status_icon, gpointer data);
void statuswindow_traypopup(GtkStatusIcon *status_icon, guint button, guint activate_time, gpointer data);
void statuswindow_traytune(GtkMenuItem *menuitem, gpointer data);
gboolean statuswindow_traytooltip(GtkStatusIcon *status_icon, gint x, gint y, gboolean keyboard_mode, GtkTooltip *tooltip, gpointer data);

static GtkWidget *statuswindow_build_window(void);
static void statuswindow_init_settings(void);
static void statuswindow_init_threads(void);

int main(int argc, char *argv[]);


gchar *mainwindow_string_definition(guint table, guint id)
{
	gchar *polarisation_tab[] = {"V","H","L","R"};
	gchar *fec_tab[] = {"AUTO","1/2","2/3","3/4","3/5","4/5","5/6","6/7","7/8","8/9","9/10","NONE"};
	gchar *system_tab[] = {"DVB-S","DVB-S2","Undefined"};
	gchar *modulation_tab[] = {"QPSK","8PSK"};
	gchar *rolloff_tab[] = {"Unknown","0.35","0.25","0.20"};
	gchar *port10_tab[] = {"0","A","B","C","D"};
	gchar *port11_tab[] = {"0","1","2","3","4","5","6","7","8"};
	
	switch (table) {
		case DEFINITION_POLARISATION:
			return polarisation_tab[id];
		case DEFINITION_FEC:
			return fec_tab[id];
		case DEFINITION_SYSTEM:
			return system_tab[id];
		case DEFINITION_MODULATION:
			return modulation_tab[id];
		case DEFINITION_ROLLOFF:
			return rolloff_tab[id];
		case DEFINITION_PORT10:
			return port10_tab[id];
		case DEFINITION_PORT11:
			return port11_tab[id];
		default: break;
	}
	
	return "Unknown";
}

gchar *mainwindow_get_client_net_address(void)
{
	return g_strdup(netgate.clientipstr);
}

static void mainwindow_tune_problem_message_response(GtkDialog *dialog, gint response_id, gpointer data)
{
	gtk_widget_destroy(GTK_WIDGET(dialog));
}

static gboolean mainwindow_tune_problem_message(gpointer data)
{
	GtkWidget *dialog;
	gint res;
	gchar *message;
	
	switch (GPOINTER_TO_INT(data)) {
		case DVBSTATUS_VALUES_IF_WRONG:
			message = _("Service frequency not supported by selected device tuner or driver");
			break;
		case DVBSTATUS_VALUES_SR_WRONG:
			message = _("Service symbol rate not supported by selected device demodulator or driver");
			break;
		default:
			message = _("Service parameters not supported by selected hardware or driver");
			break;
	}
	
	dialog = gtk_message_dialog_new(GTK_WINDOW(statuswindow.win),
									GTK_DIALOG_MODAL,
									GTK_MESSAGE_ERROR,
									GTK_BUTTONS_CLOSE,
									message,
									NULL);
	
	g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(mainwindow_tune_problem_message_response), NULL);
	gtk_widget_show(dialog);
	
	return FALSE;
}

gboolean mainwindow_tune_start(guint32 id, guchar type)
{
	struct _service *currentservice;
	gint calcres;
	
	if (type == SERVICE_NETSHARE) {
		netgate.netsvcid = id;
		g_printf("[+] Tuning to channel at server: ID:%u\n", netgate.netsvcid);
		signalthread.action = SIGNAL_THREAD_NET_TUNE;
		return TRUE;
	} else {
		dvb_demux_close(&frontend, &demuxer, svc);
		if ((currentservice = (struct _service *)service_read(id)) != NULL) {
			calcres = dvb_calculate_values(&frontend, currentservice, TRUE);
			if (calcres == DVBSTATUS_VALUES_ALL_GOOD) {
				if (svc != NULL) service_free(svc);
				svc = service_clone(currentservice);
				signalthread.action = SIGNAL_THREAD_FRONTEND_TUNE;
				return TRUE;
			} else {
				switch (calcres) {
					case DVBSTATUS_VALUES_IF_WRONG:
						g_printf("[-] Service frequency not supported by selected device tuner or driver\n");
						break;
					case DVBSTATUS_VALUES_SR_WRONG:
						g_printf("[-] Service symbol rate not supported by selected device demodulator or driver\n");
						break;
					default:
						g_printf("[-] Service parameters not supported by selected hardware or driver\n");
						break;
				}
				g_idle_add(mainwindow_tune_problem_message, GINT_TO_POINTER(calcres));
				return FALSE;
			}
		} else {
			zero_signal(NULL);
			signalthread.action = SIGNAL_THREAD_POLL;
			return FALSE;
		}
	}
}

gboolean mainwindow_network_start(gchar *address, gushort port, gchar *password)
{
	if ((address != NULL) && (password != NULL) && (port > 1024)) {
		if (udpclient_set_parameters(&netgate, address, port, password)) {
			if (svc != NULL) service_free(svc);
			svc = service_new(NULL, "Service", SERVICE_NETSHARE);
			signalthread.action = SIGNAL_THREAD_NET_INIT;
				
			return TRUE;
		}
	}
	
	return FALSE;
}

void mainwindow_network_servicelist_refresh(void)
{
	if (netgate.clientconnected)
		signalthread.action = SIGNAL_THREAD_NET_REFRESH;
}

void mainwindow_satscan_start(gboolean resume)
{ 
	if (!resume) {
		if (dvb_frontend_opened(frontend.frontend_fd)) {
			dvb_demux_close(&frontend, &demuxer, svc);
		}
		frontend.frozen = TRUE;
		signalthread.action = SIGNAL_THREAD_SATSCAN;
	} else {
		if (frontend.frozen) {
			if ((frontend.tuned) && (svc != NULL)) {
				signalthread.action = SIGNAL_THREAD_FRONTEND_TUNE;
			} else {
				zero_signal(NULL);
				signalthread.action = SIGNAL_THREAD_POLL;
			}
			frontend.frozen = FALSE;
		}
	}
}

tuneparams_t mainwindow_get_tuneparams(void)
{
	static struct _tuneparams tuneparams;
	
	if (svc == NULL) return NULL;
	
	tuneparams.freq = svc->freq;
	tuneparams.sr = svc->sr;
	tuneparams.pol = svc->pol;
	tuneparams.fec = svc->fec;
	tuneparams.sys = svc->sys;
	tuneparams.mod = svc->mod;
	tuneparams.rof = svc->rof;
	tuneparams.port10 = svc->port10;
	tuneparams.port11 = svc->port11;
	tuneparams.rep11 = svc->rep11;
	tuneparams.lnbtype = svc->lnbtype;
	tuneparams.lnb22khz = svc->lnb22khz;
	
	if (frontend.tuned) {
		tuneparams.tuned = (guchar)frontend.tuned;
		tuneparams.realfreq = frontend.realfreq;
		tuneparams.realsr = frontend.realsr;
	} else {
		tuneparams.tuned = (guchar)frontend.tuned;
		tuneparams.realfreq = 0;
		tuneparams.realsr = 0;
	} 
	
	return &tuneparams;
}

netgate_t mainwindow_get_netgate(void)
{
	return &netgate;
}

streamconfig_t mainwindow_get_streamconfig(void)
{
	return &streamconfig;
}

globalsettings_t mainwindow_get_globalsettings(void)
{
	return &globalsettings;
}

frontend_t mainwindow_get_frontend(void)
{
	return &frontend;
}

service_t mainwindow_get_service(void)
{
	return svc;
}

void mainwindow_get_signal_status(struct _sigstatus *sigstatus)
{
	if (sigstatus == NULL) return;
	
	memset(sigstatus, 0, sizeof(sigstatus));
	
	if ((!frontend.tuned) || (svc == NULL)) {
		g_snprintf(sigstatus->svcname, sizeof(sigstatus->svcname), "Unknown service");
		sigstatus->snr = 0;
		sigstatus->agc = 0;
		sigstatus->ber = 0;
		sigstatus->lock = FALSE;
		sigstatus->carrier = FALSE;
	} else {
		g_snprintf(sigstatus->svcname, sizeof(sigstatus->svcname), "%s", svc->name);
		if (!frontend.reset) {
			sigstatus->snr = frontend.snr;
			sigstatus->agc = frontend.signal;
			sigstatus->ber = frontend.ber;
			sigstatus->lock = frontend.lock;
			sigstatus->carrier = frontend.carrier;
		}
	}
} 

void mainwindow_graph_set_colors(GdkColor *mincolor, GdkColor *maxcolor)
{
	gchar *colorstr;
	
	if ((mincolor == NULL) || (maxcolor == NULL)) return;
	
	colorstr = gdk_color_to_string(mincolor);
	settings_append_string("graph_low_color", colorstr);
	g_free(colorstr);
	
	colorstr = gdk_color_to_string(maxcolor);
	settings_append_string("graph_high_color", colorstr);
	g_free(colorstr);
	
	gtk_grstatus_set_colors(statuswindow.grstatus, mincolor, maxcolor);
}

static void mainwindow_streamserver_restart_response(GtkDialog *dialog, gint response_id, gpointer data)
{
	if (response_id == GTK_RESPONSE_YES) {
		streamconfig.serverctl = streamconfig_service_restart(&streamconfig);
	}
	
	gtk_widget_destroy(GTK_WIDGET(dialog));
}

static gboolean mainwindow_streamserver_restart(gpointer data)
{
	GtkWidget *dialog;
	gint res;
	
	dialog = gtk_message_dialog_new(GTK_WINDOW(statuswindow.win),
									GTK_DIALOG_MODAL,
									GTK_MESSAGE_QUESTION,
									GTK_BUTTONS_YES_NO,
									_("Streaming server service stopped. Do you want to restart it?"),
									NULL);
	
	g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(mainwindow_streamserver_restart_response), NULL);
	gtk_widget_show(dialog);
	
	return FALSE;
}

static void mainwindow_stop_signal_handler(int sig, siginfo_t *si, void *unused)
{
	signalthread.alive = FALSE;
	soundthread.alive = FALSE;
	netstatusthread.alive = FALSE;
	settings_close();
	streamconfig_close(&streamconfig);
	if (dvb_frontend_opened(frontend.frontend_fd)) {
		dvb_demux_close(&frontend, &demuxer, svc);
		mainwindow_device_close(NULL);
	}
	printf("[-] System signal recieved: %s from %ld\n", sys_siglist[sig], si->si_pid);
	
	exit(EXIT_FAILURE);
}

static void mainwindow_service_signal_handler(int sig, siginfo_t *si, void *unused)
{
	if ((streamconfig.serverctl) && ((si->si_pid == streamconfig.processpid) || (si->si_pid == getpid()))) {
		streamconfig.serverctl = FALSE;
		printf("[-] Stream server service stopped (PID: %ld)\n", si->si_pid);
		g_idle_add(mainwindow_streamserver_restart, NULL);
	}
}

static gboolean stop_program(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	signalthread.alive = FALSE;
	soundthread.alive = FALSE;
	netstatusthread.alive = FALSE;
	streamconfig_close(&streamconfig);
	if (dvb_frontend_opened(frontend.frontend_fd)) {
		dvb_demux_close(&frontend, &demuxer, svc);
		mainwindow_device_close(NULL);
	}
	gtk_widget_destroy(statuswindow.win);
	settings_close();
	gtk_main_quit();
    
    return FALSE;
}

static gboolean close_main_window(GtkWidget *widget, GdkEvent  *event, gpointer data)
{
	if (settings_read_boolean("program_exit_on_close", FALSE)) {
		stop_program(NULL, NULL, NULL);
		return FALSE;
	} else {
		return gtk_widget_hide_on_delete(widget);
	}
}

static gboolean mainwindow_satscan_open(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	satscan_window_create(GTK_WINDOW(statuswindow.win));
	return FALSE;
}

static gboolean search_open(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	servicelist_new(GTK_WINDOW(statuswindow.win));
	return FALSE;
}

static gboolean devconfig_open(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	devwindow_create();
	return FALSE;
}

static gboolean preferences_open(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	prefwindow_create();
	return FALSE;
}

static gboolean network_open(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	netwindow_create(&netgate); 
	return FALSE;
}

static gboolean scan_open(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	scanwindow_create(frontend.adapter_nr, frontend.frontend_nr);
	return FALSE;
}

static gboolean sconf_open(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	streamwindow_create();
	return FALSE;
}

static gboolean sclients_open(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	strcliwindow_create();
	return FALSE;
}

static gboolean tsstruct_open(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	tswindow_create();
	return FALSE;
}

static gboolean calculations_open(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	calcwindow_create();
	return FALSE;
}

static gboolean mainwindow_fullscreen_open(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	fswindow_create();
	return FALSE;
}

static gboolean toggle_sound_toolbar(GtkToggleToolButton *toggle_tool_button, gpointer data)
{
	if (!statuswindow.sound_lock) {
		statuswindow.sound_lock = TRUE;
		if (gtk_toggle_tool_button_get_active(toggle_tool_button)) {
			g_print("Sounds: on\n");
			soundthread.action = SOUND_THREAD_PLAY;
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(statuswindow.sound_menu),TRUE);
		} else {
			g_print("Sounds: off\n");
			soundthread.action = SOUND_THREAD_POLL;
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(statuswindow.sound_menu),FALSE);
		}
		statuswindow.sound_lock = FALSE;
	}
	
	return FALSE;
}

static gboolean toggle_sound_menu(GtkCheckMenuItem *checkmenuitem, gpointer data)
{
	if (!statuswindow.sound_lock) {
		statuswindow.sound_lock = TRUE;
		if (gtk_check_menu_item_get_active(checkmenuitem)) {
			g_print("Sounds: on\n");
			soundthread.action = SOUND_THREAD_PLAY;
			gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(statuswindow.sound_tb),TRUE);
		} else {
			g_print("Sounds: off\n");
			soundthread.action = SOUND_THREAD_POLL;
			gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(statuswindow.sound_tb),FALSE);
		}
		statuswindow.sound_lock = FALSE;
	}
	
	return FALSE;
}

static gboolean toggle_streaming_menu(GtkCheckMenuItem *checkmenuitem, gpointer data)
{
	gint option = GPOINTER_TO_INT(data);
	gboolean value = FALSE;
	
	if (!statuswindow.streaming_lock) {
		statuswindow.streaming_lock = TRUE;
		
		switch (option) {
			case 0: {
				value = gtk_check_menu_item_get_active(checkmenuitem);
				if (streamconfig_set_record_status(&streamconfig, value)) {
					gtk_check_menu_item_set_active(checkmenuitem, value);
				}
				break;
			}
			
			case 1: {
				value = gtk_check_menu_item_get_active(checkmenuitem);
				if (streamconfig_set_udp_broadcast_status(&streamconfig, value)) {
					gtk_check_menu_item_set_active(checkmenuitem, value);
				}
				break;
			}
			
			case 2: {
				value = gtk_check_menu_item_get_active(checkmenuitem);
				if (streamconfig_set_udp_multicast_status(&streamconfig, value)) {
					gtk_check_menu_item_set_active(checkmenuitem, value);
				}
				break;
			}
			
			case 3: {
				value = gtk_check_menu_item_get_active(checkmenuitem);
				if (streamconfig_set_tcp_unicast_status(&streamconfig, value)) {
					gtk_check_menu_item_set_active(checkmenuitem, value);
				}
				break;
			}
		
			default: break;
		}
		statuswindow.streaming_lock = FALSE;
	}
		
	return FALSE;
}

static gboolean streaming_menu_init(GtkMenuItem *menuitem, gpointer data)
{
	gboolean sensitive = FALSE;
	gboolean record = FALSE, udpbc = FALSE, udpmc = FALSE, tcpuc = FALSE;
	
	if ((globalsettings.nostreaming) || (streamconfig_service_state(&streamconfig) == -1) || (!streamconfig.serverctl)) {
		sensitive = FALSE;
	} else {
		streamconfig_get_service_flags(&streamconfig, &record, &udpbc, &udpmc, &tcpuc);
		sensitive = TRUE;
	}
	
	gtk_widget_set_sensitive(statuswindow.serverclients_menu, sensitive);
	gtk_widget_set_sensitive(statuswindow.tsstruct_menu, sensitive);
	gtk_widget_set_sensitive(statuswindow.record_menu, sensitive);
	gtk_widget_set_sensitive(statuswindow.udpbc_menu, sensitive);
	gtk_widget_set_sensitive(statuswindow.udpmc_menu, sensitive);
	gtk_widget_set_sensitive(statuswindow.tcpuc_menu, sensitive);
	
	if (sensitive) {
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(statuswindow.record_menu), record);
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(statuswindow.udpbc_menu), udpbc);
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(statuswindow.udpmc_menu), udpmc);
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(statuswindow.tcpuc_menu), tcpuc);
	}
	
	return FALSE;
}

gpointer sounds_thread(gpointer data)
{
	while (soundthread.alive) {
		switch(soundthread.action) {
			case SOUND_THREAD_PLAY: { 
				sound_play(&frontend, THREAD_TIMEOUT);
				break;
			}
			
			case SOUND_THREAD_POLL: g_usleep(THREAD_TIMEOUT); break;
			
			default: g_usleep(THREAD_TIMEOUT); break;
		}
	}
	
    return NULL;
}

gpointer signal_thread(gpointer data)
{
	transponders_t transponders;
	
    while (signalthread.alive) {
		switch(signalthread.action) {
			case SIGNAL_THREAD_FRONTEND_READ: {
				dvb_status(&frontend, svc, (frontend.open_mode_readonly) ? NULL : read_event);
				if ((frontend.snr != frontend.oldsnr) && (frontend.reset)) frontend.reset = FALSE;
				g_idle_add(read_signal, NULL);
				break;
			}
			
			case SIGNAL_THREAD_FRONTEND_TUNE: 
				tune_frontend(NULL);
				break;
				
			case SIGNAL_THREAD_SATSCAN:
				if ((transponders = satscan_window_get_transponders()) != NULL) {
					if (dvb_satscan_init(&frontend, transponders)) {
						satscan_window_scan(&frontend);
					}
				}			
				
				g_idle_add(satscan_window_scan_status_fill, NULL);
				signalthread.action = SIGNAL_THREAD_POLL;
				break;
			
			case SIGNAL_THREAD_NET_INIT: {
				frontend.tuned = FALSE;
				if (!udpclient_connect(&netgate)) {
					g_idle_add(zero_signal, NULL);
					signalthread.action = SIGNAL_THREAD_POLL;
					if (netgate.error != NULL) {
						g_idle_add(message_error_nonmodal, (gpointer)netgate.error);
					}
				} else {
					frontend.reset = TRUE;
					if (!dvb_frontend_opened(frontend.frontend_fd)) {
						if (service_net_get_number() > 0) {
							g_idle_add(lock_search_buttons, GINT_TO_POINTER(FALSE));
						}
					}
					signalthread.action = SIGNAL_THREAD_NET_READ;
				}
				break;
			}
						
			case SIGNAL_THREAD_NET_READ: {
				if (netgate.clientconnected) {
					udpclient_get_signal_status(&netgate, svc, &frontend);
					
					if (frontend.reset) {
						g_idle_add(zero_signal, NULL);
						g_idle_add(mainwindow_status_message, GINT_TO_POINTER(STATUS_MESSAGE_SVC_NETSARE));
						frontend.reset = FALSE;
					}
										
					g_idle_add(read_signal, NULL);
				} else {
					g_idle_add(zero_signal, NULL);
					signalthread.action = SIGNAL_THREAD_POLL;
					if (netgate.error != NULL) {
						g_idle_add(message_error_nonmodal, (gpointer)netgate.error);
					}
				}
				break;
			}
			
			case SIGNAL_THREAD_NET_TUNE: {
				frontend.tuned = FALSE;
				if (!udpclient_tune_service(&netgate)) {
					if (netgate.error != NULL) {
						g_idle_add(message_error_nonmodal, (gpointer)netgate.error);
					}
				}
				signalthread.action = SIGNAL_THREAD_NET_READ;
				frontend.reset = TRUE;
				break;
			}
			
			case SIGNAL_THREAD_NET_REFRESH: {
				if (!udpclient_get_channels_list(&netgate)) {
					if (netgate.error != NULL) {
						g_idle_add(message_error_nonmodal, (gpointer)netgate.error);
					}
				}
				signalthread.action = SIGNAL_THREAD_NET_READ;
				break;
			}
			
			case SIGNAL_THREAD_POLL: break;
			
			default: break;
		}
		usleep(THREAD_TIMEOUT);
	}
	
    return NULL;
}

gpointer netstatus_thread(gpointer data)
{
	//udpserver_start(&netgate);
	
	communication_initialize();
	
	return NULL;
}

void mainwindow_history_tune_start(GtkMenuItem *menuitem, gpointer data)
{
	gint historyid;
	struct _historyentry *currenthe;
	
	historyid = GPOINTER_TO_INT(data);
	currenthe = &g_array_index(history, struct _historyentry, historyid);
	
	if (currenthe != NULL) mainwindow_tune_start(currenthe->id, currenthe->type);
}

static gint read_event(gint status, uint32_t freq, uint32_t sr, gint fec)
{
	gchar *statusmsg;
	GtkWidget *history_submenu;
	struct _historyentry historyentry;
	gint i;
	struct _historyentry *currenthe;
	
	if (status == 0) {
		//correct if and sr in frontend struct
		frontend.realfreq = freq/1000;
		frontend.realsr = sr/1000;
		frontend.realfec = fec;
		//set snr scale to zero
		frontend.reset = TRUE;
		frontend.oldsnr = frontend.snr;
		//no need to restart demuxers if signal is lost
		if (!frontend.tuned) {
			dvb_demux_open(&frontend, &demuxer, svc);
			streamconfig_set_stream_data(&streamconfig, svc->name, demuxer.mediastreams, svc->caid, svc->key);
		}
			
		gdk_threads_enter();
		if (!frontend.tuned) {
			lock_search_buttons(GINT_TO_POINTER(FALSE));
			lock_scan_buttons(FALSE);
			
			mainwindow_status_message(GINT_TO_POINTER(STATUS_MESSAGE_SVC));
						
			memset(historyentry.name, 0, sizeof(historyentry.name));
			sprintf(historyentry.name, "%s", svc->name);
			historyentry.id = svc->id;
			historyentry.type = svc->type;
			g_array_prepend_val(history, historyentry);
			if (historylength < 10) historylength++;
						
			if (statuswindow.historymenu_tb != NULL) gtk_widget_destroy(statuswindow.historymenu_tb);
			statuswindow.historymenu_tb = gtk_menu_new();
			
			for (i=0; i<historylength; i++) {
				if ((currenthe = &g_array_index(history, struct _historyentry, i)) != NULL) {
					history_submenu = gtk_menu_item_new_with_label(currenthe->name);
					gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.historymenu_tb), history_submenu);
					g_signal_connect(G_OBJECT(history_submenu), "activate", G_CALLBACK(mainwindow_history_tune_start), GINT_TO_POINTER(i));
					gtk_widget_show_all(history_submenu);
				}
			}
			gtk_menu_tool_button_set_menu(GTK_MENU_TOOL_BUTTON(statuswindow.search_tb), GTK_WIDGET(statuswindow.historymenu_tb));
			
			frontend.tuned = TRUE;
			//Device configuration update
			devwindow_update_page(DEV_PREFERENCE_ID_FRONTEND);
			devwindow_update_page(DEV_PREFERENCE_ID_FILTERS);
			devwindow_update_page(DEV_PREFERENCE_ID_MENU);
		}
		gdk_threads_leave();
		
	}
	
	g_printf("[+] Handled event status: %u, frequency: %u, symbolrate: %u, fec: %u\n", status, freq/1000, sr/1000, fec);
	
	return 1;
}

static gboolean read_signal(gpointer data)
{
	gchar str[30];
	gdouble level;
	
	if ((level = (gdouble)(frontend.reset ? 0 : frontend.snr)/settings_read_float("scale_snr",100.0)) > 1.0) level = 1.0;
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(statuswindow.snr), level);
	memset(str, 0, sizeof(str));
	if (frontend.lock) g_sprintf(&str, "%u [LOCK]", frontend.reset ? 0 : frontend.snr); else g_sprintf(&str, "%u", frontend.reset ? 0 : frontend.snr);
	gtk_grstatus_set_value(statuswindow.grstatus, (gchar)(level > 1.0 ? 100 : level*100));
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(statuswindow.snr), str);
	if ((level = (gdouble)((frontend.signal)/settings_read_float("scale_agc",1000.0))) > 1.0) level = 1.0;
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(statuswindow.agc), level);
	memset(str, 0, sizeof(str));
	if (frontend.carrier) g_sprintf(&str,"%u [CARRIER]", frontend.signal); else g_sprintf(&str, "%u", frontend.signal);
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(statuswindow.agc), str);
	if ((level = (gdouble)((frontend.ber)/settings_read_float("scale_ber",65535.0))) > 1.0) level = 1.0;
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(statuswindow.ber), level);
	memset(str, 0, sizeof(str));
	g_sprintf(&str, "%u", frontend.ber);
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(statuswindow.ber), str);
	
	if (svc != NULL) {
		switch (svc->type) {
			case SERVICE_TV_RADIO:
			case SERVICE_RADIO:
			case SERVICE_DATA:
			case SERVICE_TRANSPONDER:
				
				break;
			
			case SERVICE_NETSHARE:
				g_printf("[+] Server signal status: %s@%s SNR: %u, AGC: %u, BER: %u, LOCK: %u, CARRIER: %u\n", svc->name, svc->node, frontend.snr, frontend.signal, frontend.ber, frontend.lock, frontend.carrier);
				break;
			
			default: break;
		}
	}
	
	fswindow_display_signal_status(&frontend);

	return FALSE;
}

gboolean zero_signal(gpointer data)
{
	if (frontend.snr != 0) frontend.snr = 0;
	if (frontend.signal != 0) frontend.signal = 0;
	if (frontend.ber != 0) frontend.ber = 0;
	if (frontend.lock) frontend.lock = FALSE;
	if (frontend.carrier) frontend.carrier = FALSE;
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(statuswindow.snr), 0.0);
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(statuswindow.snr), " ");
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(statuswindow.agc), 0.0);
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(statuswindow.agc), " ");
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(statuswindow.ber), 0.0);
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(statuswindow.ber), " ");
		
	return FALSE;
}

gboolean mainwindow_enum_devices(gpointer data)
{
	dvbname_free_devices(&devices);
	devices = dvbname_scan_devices();
	
	return FALSE;	
}

GSList *mainwindow_get_devices(gboolean rescan)
{
	if (rescan) mainwindow_enum_devices(NULL);
	
	return devices;
}

/*static void mainwindow_device_search_foreach(gpointer data, gpointer user_data)
{
	struct _device *device;
	
	device = (struct _device *)data;
	
	
}*/

gint mainwindow_device_search(guint adapterid, guint frontendid, gboolean rescan, gboolean restart)
{
	gsize length;
	gint i;
	struct _device *device;
	gboolean devfound;
	gboolean unique;
	
	if (rescan) mainwindow_enum_devices(NULL);
	
	devfound = FALSE;
	
	if ((length = g_slist_length(devices)) > 0) {
		for (i=0; i<length; i++) {
			device = (struct _device *)g_slist_nth_data(devices, i);
			if ((adapterid == device->adapter) && (frontendid == device->frontend)) {
				devfound = TRUE;
				unique = ((device->adapter != frontend.adapter_nr) || (device->frontend != frontend.frontend_nr));
				if ((!restart) || ((restart) && (!unique))) {
					mainwindow_device_open(device);
				}
				break;
			}
		}
		
		if (!devfound) {
			device = (struct _device *)g_slist_nth_data(devices, 0);
			unique = ((device->adapter != frontend.adapter_nr) || (device->frontend != frontend.frontend_nr));
			if ((!restart) || ((restart) && (!unique))) {
				mainwindow_device_open(device);
			}
			return -(length);
		}
	}
	
	return length;
}	

static gboolean mainwindow_device_open(struct _device *device)
{
	if (device != NULL) {
		mainwindow_device_close(NULL); //if changing frontend
		mainwindow_status_message(GINT_TO_POINTER(STATUS_MESSAGE_FR_OPENING));
		frontend.adapter_nr = device->adapter;
		frontend.frontend_nr = device->frontend;
		frontend.frontend_fn = dvbname_get_frontend_filename(device->adapter, device->frontend);
		
		signalthread.action = SIGNAL_THREAD_POLL;
		
		if ((frontend.frontend_fd = dvb_open(&frontend)) < 0 ) {
			//error while opening
			lock_search_buttons(GINT_TO_POINTER(TRUE));
			lock_scan_buttons(TRUE);
			mainwindow_status_message(GINT_TO_POINTER(STATUS_MESSAGE_FR_OPEN_FAIL));
			//Device configuration update
			devwindow_update_page(DEV_PREFERENCE_ID_DRIVER);
			devwindow_update_page(DEV_PREFERENCE_ID_MENU);
			signalthread.action = SIGNAL_THREAD_POLL;
		} else {
			if (!frontend.open_mode_readonly) {
				//open in full access mode
				lock_search_buttons(GINT_TO_POINTER(FALSE));
				mainwindow_status_message(GINT_TO_POINTER(STATUS_MESSAGE_TP_SELECTION));
				//Device configuration update
				devwindow_update_page(DEV_PREFERENCE_ID_DRIVER);
				devwindow_update_page(DEV_PREFERENCE_ID_MENU);
				signalthread.action = SIGNAL_THREAD_POLL;
			} else {
				//open in readonly mode
				lock_search_buttons(GINT_TO_POINTER(TRUE));
				lock_scan_buttons(TRUE);
				//Need to clone empty service for reading parameters
				if (svc != NULL) service_free(svc);
				svc = service_new(NULL, "Readonly mode service", SERVICE_TRANSPONDER);
				dvb_parameters(&frontend, svc);
				frontend.realfec = svc->fec;
				frontend.realfreq = svc->freq;
				frontend.realsr = svc->sr;
				mainwindow_status_message(GINT_TO_POINTER(STATUS_MESSAGE_FR_READONLY_MODE));
				//Device configuration update
				devwindow_update_page(DEV_PREFERENCE_ID_DRIVER);
				devwindow_update_page(DEV_PREFERENCE_ID_MENU);
				signalthread.action = SIGNAL_THREAD_FRONTEND_READ;
			}
			//save opened device idents
			settings_append_int("adapter", device->adapter);
			settings_append_int("frontend", device->frontend);
			settings_flush();
		}
	} else {
		mainwindow_status_message(GINT_TO_POINTER(STATUS_MESSAGE_FR_NOT_FOUND));
		//Device configuration update
		devwindow_update_page(DEV_PREFERENCE_ID_DRIVER);
		devwindow_update_page(DEV_PREFERENCE_ID_MENU);
	}
	
	return FALSE;
}


static gboolean mainwindow_device_close(gpointer data)
{
	if (dvb_frontend_opened(frontend.frontend_fd)) {
		signalthread.action = SIGNAL_THREAD_POLL;
		zero_signal(NULL);
		if (dvb_close(&frontend)) frontend.frontend_fd = -1;
		zero_signal(NULL);
		frontend.tuned = FALSE;
		g_free(frontend.frontend_fn);
		frontend.frontend_nr = 0;
		frontend.adapter_nr = 0;
		memset(frontend.driver, 0, 128);
		frontend.frequency_min = 0;
		frontend.frequency_max = 0;
		frontend.frequency_stepsize = 0;
		frontend.symbol_rate_min = 0;
		frontend.symbol_rate_max = 0;
		frontend.dvbs_support = FALSE;
		frontend.dvbs2_support = FALSE;
		frontend.open_mode_readonly = FALSE;
		frontend.api_version = 0;
		frontend.locked = FALSE;
		frontend.timestamp = 0;
		frontend.lockattempts = 0;
		frontend.frozen = FALSE;
		frontend.error = NULL;
		lock_search_buttons(GINT_TO_POINTER(TRUE));
		lock_scan_buttons(TRUE);
	}
	
	return FALSE;
}

static gboolean tune_frontend(gpointer data)
{
	if (dvb_frontend_opened(frontend.frontend_fd)) {
		lock_search_buttons(GINT_TO_POINTER(TRUE));
		lock_scan_buttons(TRUE);
		mainwindow_status_message(GINT_TO_POINTER(STATUS_MESSAGE_TP_TUNING));
		if (dvb_tune(&frontend, svc)) {
			frontend.tuned = FALSE;
			signalthread.action = SIGNAL_THREAD_FRONTEND_READ;
		} else {
			zero_signal(NULL);
			signalthread.action = SIGNAL_THREAD_POLL;
			mainwindow_status_message(GINT_TO_POINTER(STATUS_MESSAGE_TP_TUNUNG_FAIL));
		}				 
	}
	
	return FALSE;
}

static gboolean mainwindow_status_message(gpointer data)
{
	guint context_id;
	gint message_id;
	gchar *message;
	
	message_id = GPOINTER_TO_INT(data);
	
	switch (message_id) {
		case STATUS_MESSAGE_SVC_NETSARE: {
			message = g_strdup_printf(_("Service %s at host %s"), svc->name, svc->node);
			fswindow_display_infobar(svc);
			break;
		}
		
		case STATUS_MESSAGE_SVC: {
			switch (svc->type) {
				case SERVICE_TV_RADIO: {
					message = g_strdup_printf(_("Service %s"), svc->name);
					break;
				}
				case SERVICE_RADIO: {
					message = g_strdup_printf(_("Radio service %s"), svc->name);
					break;
				}
				case SERVICE_DATA: {
					message = g_strdup_printf(_("Data service %s"), svc->name);
					break;
				}
				case SERVICE_TRANSPONDER: {
					message = g_strdup_printf(_("Transponder %s"), svc->name);
					break;
				}
				case SERVICE_NETSHARE: {
					message = g_strdup_printf(_("Service %s at host %s"), svc->name, svc->node);
					break;
				}
				default: {
					message = g_strdup_printf(_("Service %s"), svc->name);
					break;
				}
			}
			fswindow_display_infobar(svc);
			break;
		}
		
		case STATUS_MESSAGE_FR_OPENING: {
			message = g_strdup(_("Opening frontend..."));
			break;
		}
		
		case STATUS_MESSAGE_FR_OPEN_FAIL: {
			message = g_strdup(_("Failed to open frontend"));
			break;
		}
		
		case STATUS_MESSAGE_TP_SELECTION: {
			message = g_strdup(_("Select transponder to tune..."));
			break;
		}
		
		case STATUS_MESSAGE_FR_READONLY_MODE: {
			message = g_strdup(_("Device opened in readonly mode..."));
			fswindow_display_infobar(svc);
			break;
		}
		
		case STATUS_MESSAGE_FR_NOT_FOUND: {
			message = g_strdup(_("Unable to find any frontend to open"));
			break;
		}
		
		case STATUS_MESSAGE_TP_TUNING: {
			message = g_strdup(_("Tuning to transponder..."));
			break;
		}
		
		case STATUS_MESSAGE_TP_TUNUNG_FAIL: {
			message = g_strdup(_("Failed to tune transponder"));
			break;
		}
		
		case STATUS_MESSAGE_DEV_NOT_FOUND: {
			message = g_strdup(_("No devices found in system..."));
			break;
		}
		
		default: break;
	}
	
	if (message != NULL) {
		context_id = gtk_statusbar_get_context_id(GTK_STATUSBAR(statuswindow.statusbar), message);
		gtk_statusbar_push(GTK_STATUSBAR(statuswindow.statusbar), context_id, message);	
		g_free(message);
	}
	
	return FALSE;
}

static gboolean lock_sound_buttons(gboolean lock)
{
	if (lock) {
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.sound_tb),FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.sound_menu),FALSE);
	} else {
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.sound_tb),TRUE);
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.sound_menu),TRUE);
	}
   	
	return FALSE;
}

gboolean lock_search_buttons(gpointer data)
{
	gboolean lock = GPOINTER_TO_INT(data);
	
	if (lock) {
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.search_tb), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.search_menu), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.search_tm), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.services_tm), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.satscan_menu), FALSE);
	} else {
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.search_tb), TRUE);
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.search_menu), TRUE);
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.search_tm), TRUE);
		if (dvb_frontend_opened(frontend.frontend_fd)) {
			gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.services_tm), TRUE);
			gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.satscan_menu), TRUE);
		}
	}
   	
	return FALSE;
}

static gboolean lock_scan_buttons(gboolean lock)
{
	if (lock) {
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.scan_menu),FALSE);
	} else {
		gtk_widget_set_sensitive(GTK_WIDGET(statuswindow.scan_menu),TRUE);
	}
   	
	return FALSE;
}

void statuswindow_trayaction(GtkStatusIcon *status_icon, gpointer data)
{
	#if GTK_CHECK_VERSION(2, 20, 0)
		if (gtk_widget_get_visible(statuswindow.win)) {
	#else
		if (GTK_WIDGET_VISIBLE(statuswindow.win)) {
	#endif
		#if GTK_CHECK_VERSION(2, 24, 0)
			gtk_widget_hide(statuswindow.win);
		#else
			gtk_widget_hide_all(statuswindow.win);
		#endif
	} else {
		gtk_widget_show_all(statuswindow.win);
	}
}

gboolean mainwindow_servicemenu_add_service_cb(service *currentsvc)
{
	statuswindow.entry_sm = gtk_menu_item_new_with_label(currentsvc->name);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.servicesmenu), statuswindow.entry_sm);
	if (svc != NULL) {
		if (currentsvc->id == svc->id) {
			gtk_widget_set_sensitive(statuswindow.entry_sm, FALSE);
		}
	}
	g_signal_connect(G_OBJECT(statuswindow.entry_sm), "activate", G_CALLBACK(statuswindow_traytune), GUINT_TO_POINTER(currentsvc->id));
	gtk_widget_show_all(statuswindow.entry_sm);
}

void statuswindow_traypopup(GtkStatusIcon *status_icon, guint button, guint activate_time, gpointer data)
{
	if (statuswindow.servicesmenu != NULL) gtk_widget_destroy(statuswindow.servicesmenu);
	
	statuswindow.servicesmenu = gtk_menu_new();
		
	service_read_foreach(mainwindow_servicemenu_add_service_cb, NULL, NULL);
	
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(statuswindow.services_tm), statuswindow.servicesmenu);
	gtk_menu_popup(GTK_MENU(data), NULL, NULL, gtk_status_icon_position_menu, status_icon, button, activate_time);
}

void statuswindow_traytune(GtkMenuItem *menuitem, gpointer data)
{
	guint32 currensvcid;
	service *currentsvc;
	
	currensvcid = GPOINTER_TO_UINT(data);
	currentsvc = service_read(currensvcid);
	
	if (currentsvc != NULL) {
		mainwindow_tune_start(currentsvc->id, currentsvc->type);
	}
}
	
gboolean statuswindow_traytooltip(GtkStatusIcon *status_icon, gint x, gint y, gboolean keyboard_mode, GtkTooltip *tooltip, gpointer data)
{
	gchar *status;
	gint level;
	
	if ((level = round((frontend.reset ? 0 : frontend.snr)*100/(settings_read_int("snr_scale",100)))) > 100) level = 100;
	
	status = g_strdup_printf(_("<b>Signal level: %i%%</b>\n<small>Transponder: %s\nSNR: %i %s\nAGC: %i %s\nBER: %i</small>"), level, (frontend.tuned) ? svc->name : _("Not tuned"), frontend.snr, (frontend.lock) ? "[LOCK]" : "", frontend.signal, (frontend.carrier) ? "[CARRIER]" : "", frontend.ber);
	gtk_tooltip_set_markup(tooltip, status);
	g_free(status);
		
	return TRUE;
} 

static GtkWidget *statuswindow_build_window(void)
{
	GtkWidget *vbox, *hbox;
	GtkWidget *align;
	GtkWidget *table;
	GtkWidget *name_l, *group_l, *state_l, *history_l;
	GtkWidget *image_menu, *image_tb;
	GdkColor lowcolor, hicolor;
	gchar *lowcolorstr, *hicolorstr;
	GError *error = NULL;
	#if GTK_CHECK_VERSION(3, 0, 0)
		GtkStyleContext *context;
	#endif
	
	statuswindow.win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	statuswindow.title = _("Signal status - GSF");
	gtk_window_set_position(GTK_WINDOW(statuswindow.win), GTK_WIN_POS_CENTER);
	gtk_window_set_title(GTK_WINDOW(statuswindow.win), statuswindow.title);
	//gtk_widget_set_size_request(statuswindow.win, 490, 240);
	gtk_window_set_default_size(GTK_WINDOW(statuswindow.win), 500, 250);
	if (!gtk_window_set_icon_from_file(GTK_WINDOW(statuswindow.win), RESOURCE_MAINWINDOW_MAINICON, &error)) {
		if (error != NULL) {
			g_printf("[-] Unable to set window icon: %s\n", error->message);
			g_error_free(error);
		}
	}
	
	vbox = gtk_vbox_new(FALSE, 0);
	//Menu bar
	statuswindow.menubar = gtk_menu_bar_new();
	statuswindow.actionsmenu = gtk_menu_new();
	
	statuswindow.accel_group = gtk_accel_group_new();
	gtk_window_add_accel_group(GTK_WINDOW(statuswindow.win), statuswindow.accel_group);
	
	statuswindow.actions_menu = gtk_menu_item_new_with_mnemonic(_("_Actions"));
	statuswindow.search_menu = gtk_image_menu_item_new_with_label(_("Tune"));
	statuswindow.network_menu = gtk_image_menu_item_new_with_label(_("Network status"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_NETWORK, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.network_menu), image_menu);
	image_menu = gtk_image_new_from_stock(GTK_STOCK_APPLY, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.search_menu), image_menu);
	statuswindow.scan_menu = gtk_image_menu_item_new_with_label(_("Transponder scan"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_FIND, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.scan_menu), image_menu);
	statuswindow.satscan_menu = gtk_image_menu_item_new_with_label(_("Satellite scan"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_SORT_ASCENDING, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.satscan_menu), image_menu);
	statuswindow.sep_menu = gtk_separator_menu_item_new();
	statuswindow.quit_menu = gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, NULL);
	
	statuswindow.editmenu = gtk_menu_new();
	statuswindow.edit_menu = gtk_menu_item_new_with_mnemonic(_("_Edit"));
	statuswindow.preferences_menu = gtk_image_menu_item_new_with_label(_("Preferences"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_PREFERENCES, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.preferences_menu), image_menu);
	statuswindow.device_menu = gtk_image_menu_item_new_with_label(_("Device"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_JUMP_TO, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.device_menu), image_menu);
	statuswindow.sound_menu = gtk_check_menu_item_new_with_label(_("Enable sound"));
	
	statuswindow.viewmenu = gtk_menu_new();
	statuswindow.view_menu = gtk_menu_item_new_with_mnemonic(_("_View"));
	statuswindow.fullscreen_menu = gtk_image_menu_item_new_with_label(_("Fullscreen"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_FULLSCREEN, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.fullscreen_menu), image_menu);
	
	statuswindow.streamingmenu = gtk_menu_new();
	statuswindow.streaming_menu = gtk_menu_item_new_with_mnemonic(_("_Streaming"));
	statuswindow.serverconfig_menu = gtk_image_menu_item_new_with_label(_("Configure server"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_EXECUTE, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.serverconfig_menu), image_menu);
	statuswindow.serverclients_menu = gtk_image_menu_item_new_with_label(_("Streaming clients"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_CONNECT, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.serverclients_menu), image_menu);
	statuswindow.tsstruct_menu = gtk_image_menu_item_new_with_label(_("TS structure"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.tsstruct_menu), image_menu);
	statuswindow.record_menu = gtk_check_menu_item_new_with_label(_("Enable record"));
	statuswindow.udpbc_menu = gtk_check_menu_item_new_with_label(_("Enable UDP Broadcast"));
	statuswindow.udpmc_menu = gtk_check_menu_item_new_with_label(_("Enable UDP Multicast"));
	statuswindow.tcpuc_menu = gtk_check_menu_item_new_with_label(_("Enable TCP Unicast"));
	
	statuswindow.helpmenu = gtk_menu_new();
	statuswindow.help_menu = gtk_menu_item_new_with_mnemonic(_("_Help"));
	statuswindow.calc_menu = gtk_image_menu_item_new_with_label(_("Calculations"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_CONVERT, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.calc_menu), image_menu);
	statuswindow.about_menu = gtk_image_menu_item_new_with_label(_("About"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_ABOUT, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.about_menu), image_menu);
	
	#if GTK_CHECK_VERSION(2, 22, 0)
		gtk_widget_add_accelerator(statuswindow.quit_menu, "activate", statuswindow.accel_group,
									GDK_KEY_q, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.search_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_n, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.network_menu, "activate", statuswindow.accel_group,
									GDK_KEY_c, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.scan_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_f, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.satscan_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_g, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.preferences_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_p, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.device_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_d, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.fullscreen_menu, "activate", statuswindow.accel_group,
									GDK_KEY_F11, 0, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.sound_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_s, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.serverconfig_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_e, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.serverclients_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_l, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.tsstruct_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_t, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);						 
		gtk_widget_add_accelerator(statuswindow.record_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_r, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.udpbc_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_b, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.udpmc_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_m, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.tcpuc_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_u, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.about_menu, "activate", statuswindow.accel_group, 
									GDK_KEY_h, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	#else
		gtk_widget_add_accelerator(statuswindow.quit_menu, "activate", statuswindow.accel_group,
									GDK_q, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.search_menu, "activate", statuswindow.accel_group, 
									GDK_n, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.network_menu, "activate", statuswindow.accel_group,
									GDK_c, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.scan_menu, "activate", statuswindow.accel_group, 
									GDK_f, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.satscan_menu, "activate", statuswindow.accel_group, 
									GDK_g, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.preferences_menu, "activate", statuswindow.accel_group, 
									GDK_p, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.device_menu, "activate", statuswindow.accel_group, 
									GDK_d, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.fullscreen_menu, "activate", statuswindow.accel_group,
									GDK_F11, 0, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.sound_menu, "activate", statuswindow.accel_group, 
									GDK_s, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.serverconfig_menu, "activate", statuswindow.accel_group, 
									GDK_e, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.serverclients_menu, "activate", statuswindow.accel_group, 
									GDK_l, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.tsstruct_menu, "activate", statuswindow.accel_group, 
									GDK_t, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);						 
		gtk_widget_add_accelerator(statuswindow.record_menu, "activate", statuswindow.accel_group, 
									GDK_r, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.udpbc_menu, "activate", statuswindow.accel_group, 
									GDK_b, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.udpmc_menu, "activate", statuswindow.accel_group, 
									GDK_m, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.tcpuc_menu, "activate", statuswindow.accel_group, 
									GDK_u, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
		gtk_widget_add_accelerator(statuswindow.about_menu, "activate", statuswindow.accel_group, 
									GDK_h, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	#endif
	
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(statuswindow.actions_menu), statuswindow.actionsmenu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.actionsmenu), statuswindow.search_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.actionsmenu), statuswindow.network_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.actionsmenu), statuswindow.scan_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.actionsmenu), statuswindow.satscan_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.actionsmenu), statuswindow.sep_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.actionsmenu), statuswindow.quit_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.menubar), statuswindow.actions_menu);
	
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(statuswindow.edit_menu), statuswindow.editmenu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.editmenu), statuswindow.preferences_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.editmenu), statuswindow.device_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.editmenu), statuswindow.sound_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.menubar), statuswindow.edit_menu);
	
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(statuswindow.view_menu), statuswindow.viewmenu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.viewmenu), statuswindow.fullscreen_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.menubar), statuswindow.view_menu);
	
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(statuswindow.streaming_menu), statuswindow.streamingmenu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.streamingmenu), statuswindow.serverconfig_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.streamingmenu), statuswindow.serverclients_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.streamingmenu), statuswindow.tsstruct_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.streamingmenu), statuswindow.record_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.streamingmenu), statuswindow.udpbc_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.streamingmenu), statuswindow.udpmc_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.streamingmenu), statuswindow.tcpuc_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.menubar), statuswindow.streaming_menu);
	
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(statuswindow.help_menu), statuswindow.helpmenu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.helpmenu), statuswindow.calc_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.helpmenu), statuswindow.about_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.menubar), statuswindow.help_menu);
	
	g_signal_connect(G_OBJECT(statuswindow.quit_menu), "activate", G_CALLBACK(stop_program), NULL);
	g_signal_connect(G_OBJECT(statuswindow.search_menu), "activate", G_CALLBACK(search_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.network_menu), "activate", G_CALLBACK(network_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.scan_menu), "activate", G_CALLBACK(scan_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.satscan_menu), "activate", G_CALLBACK(mainwindow_satscan_open), NULL);
		  
	g_signal_connect(G_OBJECT(statuswindow.device_menu), "activate", G_CALLBACK(devconfig_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.preferences_menu), "activate", G_CALLBACK(preferences_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.fullscreen_menu), "activate", G_CALLBACK(mainwindow_fullscreen_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.sound_menu), "toggled", G_CALLBACK(toggle_sound_menu), NULL);
	
	g_signal_connect(G_OBJECT(statuswindow.streaming_menu), "activate", G_CALLBACK(streaming_menu_init), NULL);
	g_signal_connect(G_OBJECT(statuswindow.serverconfig_menu), "activate", G_CALLBACK(sconf_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.serverclients_menu), "activate", G_CALLBACK(sclients_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.tsstruct_menu), "activate", G_CALLBACK(tsstruct_open), NULL);
		
	g_signal_connect(G_OBJECT(statuswindow.record_menu), "toggled", G_CALLBACK(toggle_streaming_menu), GINT_TO_POINTER(0));
	g_signal_connect(G_OBJECT(statuswindow.udpbc_menu), "toggled", G_CALLBACK(toggle_streaming_menu), GINT_TO_POINTER(1));
	g_signal_connect(G_OBJECT(statuswindow.udpmc_menu), "toggled", G_CALLBACK(toggle_streaming_menu), GINT_TO_POINTER(2));
	g_signal_connect(G_OBJECT(statuswindow.tcpuc_menu), "toggled", G_CALLBACK(toggle_streaming_menu), GINT_TO_POINTER(3));
	
	g_signal_connect(G_OBJECT(statuswindow.calc_menu), "activate", G_CALLBACK(calculations_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.about_menu), "activate", G_CALLBACK(dialog_about), NULL);
	gtk_box_pack_start(GTK_BOX(vbox), statuswindow.menubar, FALSE, FALSE, 0);

	//Toolbar
	statuswindow.toolbar = GTK_TOOLBAR(gtk_toolbar_new());
	image_tb = gtk_image_new_from_stock(GTK_STOCK_APPLY, GTK_ICON_SIZE_LARGE_TOOLBAR);
	statuswindow.historymenu_tb = gtk_menu_new();
	statuswindow.search_tb = gtk_menu_tool_button_new(image_tb,_("Tune"));
	gtk_widget_set_tooltip_text(GTK_WIDGET(statuswindow.search_tb), _("Tune transponder"));
    g_object_set(statuswindow.search_tb, "is-important", TRUE, NULL);
  	//gtk_menu_tool_button_set_menu(GTK_MENU_TOOL_BUTTON(statuswindow.search_tb), GTK_WIDGET(statuswindow.historymenu_tb));
  	image_tb = gtk_image_new_from_stock(GTK_STOCK_JUMP_TO, GTK_ICON_SIZE_LARGE_TOOLBAR);
	statuswindow.device_tb = gtk_tool_button_new(image_tb,_("Device"));
	gtk_widget_set_tooltip_text(GTK_WIDGET(statuswindow.device_tb), _("DVB device parameters"));
	image_tb = gtk_image_new_from_stock(GTK_STOCK_PREFERENCES, GTK_ICON_SIZE_LARGE_TOOLBAR);
	statuswindow.preferences_tb = gtk_tool_button_new(image_tb, _("Configure"));
	gtk_widget_set_tooltip_text(GTK_WIDGET(statuswindow.preferences_tb), _("Configure program"));
	statuswindow.separator1_tb = gtk_separator_tool_item_new();
	image_tb = gtk_image_new_from_stock(GTK_STOCK_MEDIA_PLAY, GTK_ICON_SIZE_LARGE_TOOLBAR);
	statuswindow.sound_tb = gtk_toggle_tool_button_new();
	gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(statuswindow.sound_tb),image_tb);
	gtk_tool_button_set_label(GTK_TOOL_BUTTON(statuswindow.sound_tb),_("Sound"));
	gtk_widget_set_tooltip_text(GTK_WIDGET(statuswindow.sound_tb), _("Enable/disable sound"));
	g_object_set(statuswindow.sound_tb, "is-important", TRUE, NULL);
	image_tb = gtk_image_new_from_stock(GTK_STOCK_FULLSCREEN, GTK_ICON_SIZE_LARGE_TOOLBAR);
	statuswindow.fullscreen_tb = gtk_tool_button_new(image_tb, _("Fullscreen"));
	gtk_widget_set_tooltip_text(GTK_WIDGET(statuswindow.fullscreen_tb), _("Fullscreen mode"));
	image_tb = gtk_image_new_from_stock(GTK_STOCK_QUIT, GTK_ICON_SIZE_LARGE_TOOLBAR);
	statuswindow.exit_tb = gtk_tool_button_new(image_tb, _("Exit"));
	gtk_widget_set_tooltip_text(GTK_WIDGET(statuswindow.exit_tb), _("Exit program"));
	statuswindow.separator2_tb = gtk_separator_tool_item_new();
	
	gtk_toolbar_insert(GTK_TOOLBAR(statuswindow.toolbar),statuswindow.search_tb, 0);
	gtk_toolbar_insert(GTK_TOOLBAR(statuswindow.toolbar),statuswindow.device_tb, 1);
	gtk_toolbar_insert(GTK_TOOLBAR(statuswindow.toolbar),statuswindow.preferences_tb, 2);
	gtk_toolbar_insert(GTK_TOOLBAR(statuswindow.toolbar),statuswindow.fullscreen_tb, 3);
	gtk_toolbar_insert(GTK_TOOLBAR(statuswindow.toolbar),statuswindow.separator1_tb, 4);
	gtk_toolbar_insert(GTK_TOOLBAR(statuswindow.toolbar),statuswindow.sound_tb, 5);
	gtk_toolbar_insert(GTK_TOOLBAR(statuswindow.toolbar),statuswindow.separator2_tb, 6);
	gtk_toolbar_insert(GTK_TOOLBAR(statuswindow.toolbar),statuswindow.exit_tb, 7);
	
	g_signal_connect(G_OBJECT(statuswindow.search_tb), "clicked", G_CALLBACK(search_open), NULL);
	//g_signal_connect(G_OBJECT(statuswindow.search_tb), "show-menu", G_CALLBACK(mainwindow_show_history_menu), NULL);
	g_signal_connect(G_OBJECT(statuswindow.device_tb), "clicked", G_CALLBACK(devconfig_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.preferences_tb), "clicked", G_CALLBACK(preferences_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.sound_tb), "toggled", G_CALLBACK(toggle_sound_toolbar), NULL);
	g_signal_connect(G_OBJECT(statuswindow.fullscreen_tb), "clicked", G_CALLBACK(mainwindow_fullscreen_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.exit_tb), "clicked", G_CALLBACK(stop_program), NULL);
	
	gtk_toolbar_set_show_arrow(GTK_TOOLBAR(statuswindow.toolbar),TRUE);
	gtk_toolbar_set_style(GTK_TOOLBAR(statuswindow.toolbar), GTK_TOOLBAR_BOTH_HORIZ);
	
	#if GTK_CHECK_VERSION(3, 0, 0)
		context = gtk_widget_get_style_context(GTK_WIDGET(statuswindow.toolbar));
		gtk_style_context_add_class(context, GTK_STYLE_CLASS_PRIMARY_TOOLBAR);
	#endif
	
	gtk_box_pack_start(GTK_BOX(vbox),GTK_WIDGET(statuswindow.toolbar),FALSE,FALSE,0);
	
	//Progressbars
	hbox = gtk_hbox_new(FALSE, 10);
	name_l = gtk_label_new(" ");
	gtk_label_set_markup(GTK_LABEL(name_l),"<b>SNR:</b>");
	gtk_box_pack_start(GTK_BOX(hbox), name_l,FALSE,FALSE,10);
	
	statuswindow.snr = gtk_progress_bar_new();
	gtk_box_pack_start(GTK_BOX(hbox), statuswindow.snr,TRUE,TRUE,10);
	gtk_box_pack_start(GTK_BOX(vbox), hbox,TRUE,TRUE,10);
	
	hbox = gtk_hbox_new(FALSE, 10);
	group_l = gtk_label_new("");
  	gtk_label_set_markup(GTK_LABEL(group_l),"<b>AGC:</b>");
	gtk_box_pack_start(GTK_BOX(hbox),group_l,FALSE,FALSE,10);
	
	statuswindow.agc = gtk_progress_bar_new();
	gtk_box_pack_start(GTK_BOX(hbox),statuswindow.agc,TRUE,TRUE,10);
	gtk_box_pack_start(GTK_BOX(vbox),hbox,TRUE,TRUE,10);
	
	hbox = gtk_hbox_new(FALSE, 10);
	state_l = gtk_label_new("");
	gtk_label_set_markup(GTK_LABEL(state_l),"<b>BER:</b>");
	gtk_box_pack_start(GTK_BOX(hbox),state_l,FALSE,FALSE,10);
	
	statuswindow.ber = gtk_progress_bar_new();
	gtk_box_pack_start(GTK_BOX(hbox),statuswindow.ber,TRUE,TRUE,10);
	gtk_box_pack_start(GTK_BOX(vbox),hbox,TRUE,TRUE,10);
	
	statuswindow.statusbar = GTK_STATUSBAR(gtk_statusbar_new());
	#if GTK_CHECK_VERSION(3, 0, 0)
		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(statuswindow.snr), " ");
		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(statuswindow.agc), " ");
		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(statuswindow.ber), " ");
		gtk_progress_bar_set_show_text(GTK_PROGRESS_BAR(statuswindow.snr), TRUE);
		gtk_progress_bar_set_show_text(GTK_PROGRESS_BAR(statuswindow.agc), TRUE);
		gtk_progress_bar_set_show_text(GTK_PROGRESS_BAR(statuswindow.ber), TRUE);
	#else
		gtk_statusbar_set_has_resize_grip(statuswindow.statusbar, FALSE);
	#endif
	
	statuswindow.grstatus = GTK_GRSTATUS(gtk_grstatus_new(100));
	lowcolorstr = settings_read_string("graph_low_color", "white");
	gdk_color_parse(lowcolorstr, &lowcolor);
	g_free(lowcolorstr);
	hicolorstr = settings_read_string("graph_high_color","green3");
	gdk_color_parse(hicolorstr, &hicolor);
	g_free(hicolorstr);
	gtk_grstatus_set_colors(statuswindow.grstatus, &lowcolor, &hicolor);
	
	statuswindow.statussep = gtk_vseparator_new();
	gtk_box_pack_start(GTK_BOX(statuswindow.statusbar), GTK_WIDGET(statuswindow.statussep), FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(statuswindow.statusbar), GTK_WIDGET(statuswindow.grstatus), FALSE, FALSE, 10);
	
	gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(statuswindow.statusbar), FALSE, FALSE, 5);
	g_signal_connect(G_OBJECT(statuswindow.win), "delete-event", G_CALLBACK(close_main_window), NULL);
	
	gtk_container_add(GTK_CONTAINER(statuswindow.win),vbox);
	statuswindow.trayicon = gtk_status_icon_new_from_file(RESOURCE_MAINWINDOW_MAINICON);
	g_signal_connect(G_OBJECT(statuswindow.trayicon), "activate",
						G_CALLBACK(statuswindow_trayaction), G_OBJECT(statuswindow.trayicon));
	statuswindow.traymenu = gtk_menu_new();
	statuswindow.search_tm = gtk_image_menu_item_new_with_label(_("Tune"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_APPLY, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.search_tm), image_menu);
	statuswindow.services_tm = gtk_image_menu_item_new_with_label(_("Services"));
	image_menu = gtk_image_new_from_stock(GTK_STOCK_DIRECTORY, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(statuswindow.services_tm), image_menu);
	
	statuswindow.sep_tm = gtk_separator_menu_item_new();
	statuswindow.quit_tm = gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, NULL);
	
	g_signal_connect(G_OBJECT(statuswindow.search_tm), "activate", G_CALLBACK(search_open), NULL);
	g_signal_connect(G_OBJECT(statuswindow.quit_tm), "activate", G_CALLBACK(stop_program), NULL);
	
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.traymenu), statuswindow.search_tm);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.traymenu), statuswindow.services_tm);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.traymenu), statuswindow.sep_tm);
	gtk_menu_shell_append(GTK_MENU_SHELL(statuswindow.traymenu), statuswindow.quit_tm);
	
	gtk_widget_show_all(statuswindow.traymenu);
	g_signal_connect(G_OBJECT(statuswindow.trayicon), "popup-menu", G_CALLBACK(statuswindow_traypopup), G_OBJECT(statuswindow.traymenu));
	
	#if GTK_CHECK_VERSION(2, 16, 0)
		gtk_status_icon_set_has_tooltip(statuswindow.trayicon, TRUE);
		g_signal_connect(G_OBJECT(statuswindow.trayicon), "query-tooltip", G_CALLBACK(statuswindow_traytooltip), G_OBJECT(statuswindow.trayicon));
	#endif
	
	lock_search_buttons(GINT_TO_POINTER(TRUE));
	lock_scan_buttons(TRUE);
	
	history = g_array_sized_new(FALSE, TRUE, sizeof(struct _historyentry), 10);
	
	return statuswindow.win;
}

static void statuswindow_init_settings(void)
{
	settings_open(FALSE);
	
	udpclient_init(&netgate);
	
	service_storage_init();
	
	fswindow_init();
	
	if (!globalsettings.nostreaming)
		streamconfig_init(&streamconfig);
}

static void statuswindow_init_threads(void)
{
	soundthread.action = SOUND_THREAD_POLL;
	soundthread.alive = TRUE;
	soundthread.thread = g_thread_create(sounds_thread, NULL, FALSE, NULL);
	
	signalthread.action = SIGNAL_THREAD_POLL;
	signalthread.alive = TRUE;
	signalthread.data = NULL;
	signalthread.thread = g_thread_create(signal_thread, NULL, FALSE, NULL);
	
	if (!globalsettings.nonetwork) {
		netstatusthread.action = SIGNAL_THREAD_POLL;
		netstatusthread.alive = TRUE;
		netstatusthread.data = NULL;
		netstatusthread.thread = g_thread_create(netstatus_thread, NULL, FALSE, NULL);
	}
	
	frontend.adapter_nr = 0;
	frontend.frontend_nr = 0;
	frontend.frontend_fd = -1;
	demuxer.netif_nr = -1;
	
	if (mainwindow_device_search(settings_read_int("adapter", 0), settings_read_int("frontend", 0), TRUE, FALSE) == 0) { 
		mainwindow_status_message(GINT_TO_POINTER(STATUS_MESSAGE_DEV_NOT_FOUND));
	}
	
	if (!sound_init()) lock_sound_buttons(TRUE);
}

#if GTK_CHECK_VERSION(3, 0, 0)
static void statuswindow_activate(GtkApplication *app)
{
	GList *list;
	GtkWidget *window;
	
	list = gtk_application_get_windows(app);
	
	if (list) {
		gtk_window_present(GTK_WINDOW(g_list_nth(list, 0)));
	} else {
		window = statuswindow_build_window();
		gtk_window_set_application(GTK_WINDOW(window), app);
		if (!settings_read_boolean("program_start_in_tray", FALSE)) {
			gtk_widget_show_all(window);
			gtk_window_set_focus(GTK_WINDOW(window), NULL);
		}
		statuswindow_init_threads();
	}
}
#endif

int main(int argc, char *argv[])
{
	struct sigaction sa;
	sigset_t sigmask;
	gint status;
	GOptionContext *context;
	GError *error = NULL;
	#if GTK_CHECK_VERSION(3, 0, 0)
		GtkApplication *app;
	#else
		GtkWidget *window;
	#endif
	
	static GOptionEntry entries[] = {
		{ "nonetwork", 'n', 0, G_OPTION_ARG_NONE, &globalsettings.nonetwork, "Disable network and web interfaces", NULL },
		{ "nostreaming", 's', 0, G_OPTION_ARG_NONE, &globalsettings.nostreaming, "Disable streaming support", NULL },
		{ NULL }
	};
	
	#ifndef LC_ALL
		#define LC_ALL 0
	#endif
	
	setlocale(LC_ALL, "");
	bindtextdomain(GSF_GETTEXT_PACKAGE, RESOURCE_DIR_LOCALE);
	bind_textdomain_codeset(GSF_GETTEXT_PACKAGE, "UTF-8");
	textdomain(GSF_GETTEXT_PACKAGE);
	
	g_set_application_name("GSF software satfinder");
	
	context = g_option_context_new("- tune satellite antenna");
	g_option_context_add_main_entries(context, entries, GSF_GETTEXT_PACKAGE);
	g_option_context_add_group(context, gtk_get_option_group(TRUE));
	
	if (!g_option_context_parse(context, &argc, &argv, &error)) {
		g_print("[-] Command line option parsing failed: %s\n", error->message);
		g_error_free(error);
		exit(EXIT_FAILURE);
	}
	
	sa.sa_flags = SA_SIGINFO;
	sigemptyset(&sa.sa_mask);
	sa.sa_sigaction = mainwindow_stop_signal_handler;
	
	sigaction(SIGTERM, &sa, NULL);
	sigaction(SIGINT, &sa, NULL);
	sigaction(SIGSEGV, &sa, NULL);
	
	sa.sa_flags = SA_SIGINFO;
	sigemptyset(&sa.sa_mask);
	sa.sa_sigaction = mainwindow_service_signal_handler;
	
	sigaction(SIGCHLD, &sa, NULL);
	
	g_thread_init(NULL);
	gdk_threads_init();
	
	statuswindow_init_settings();
	
	#if GTK_CHECK_VERSION(3, 0, 0)
		app = gtk_application_new("org.gtk.GSF", 0);
		g_signal_connect(app, "activate", G_CALLBACK(statuswindow_activate), NULL);
	#else
		gtk_init(&argc, &argv);
		window = statuswindow_build_window();
		if (!settings_read_boolean("program_start_in_tray", FALSE)) {
			gtk_widget_show_all(window);
			gtk_window_set_focus(GTK_WINDOW(window), NULL);
		}
		statuswindow_init_threads();
	#endif
	
	status = 0;
	
	gdk_threads_enter();
	#if GTK_CHECK_VERSION(3, 0, 0)
		status = g_application_run(G_APPLICATION(app), argc, argv);
	#else
		gtk_main();
	#endif
	gdk_threads_leave();
	
	#if GTK_CHECK_VERSION(3, 0, 0)
		g_object_unref(app);
	#endif
	
	return status;
}
