/*
 *      mainwindow.h
 *      
 *      Copyright 2009 - 2010 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __MAINWINDOW_H__
#define __MAINWINDOW_H__

#include <gtk/gtk.h>
#include <stdint.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

enum {
	DEFINITION_POLARISATION = 0,
	DEFINITION_FEC,
	DEFINITION_SYSTEM,
	DEFINITION_MODULATION,
	DEFINITION_ROLLOFF,
	DEFINITION_PORT10,
	DEFINITION_PORT11
};

struct _device {
	//devices GSList entry
	gchar name[256];
	gchar vendor[256];
	gchar driver[256];
	gchar slot[256];
	guint bus;
	guint vendorid;
	guint deviceid;
	gint value1;
	gint value2;
	gint adapter;
	gint frontend;
};

struct _frontend {
	//frontend params
	gchar *frontend_fn;
	gint frontend_fd;
	gint adapter_nr;
	gint frontend_nr;
	gchar driver[128];
	uint32_t frequency_min;
	uint32_t frequency_max;
	uint32_t frequency_stepsize;
	uint32_t symbol_rate_min;
	uint32_t symbol_rate_max;
	gboolean dvbs_support;
	gboolean dvbs2_support;
	gboolean open_mode_readonly;
	uint16_t api_version;
	//signal status
	uint16_t snr;
	uint16_t signal;
	uint32_t ber;
	gboolean lock;
	gboolean carrier;
	gboolean tuned;
	//fix for diseqc fail
	gboolean locked;
	time_t   timestamp;
	guint    lockattempts;
	//fix for multiple tunes
	gboolean reset;
	uint16_t oldsnr;
	gboolean frozen;
	//calculated values
	guint32  ifreq;
	gboolean conttone;
	//corrected parametes
	guint32 realfreq;
	guint32 realsr;
	guint8  realfec;
	//generic
	GError  *error;
};

struct _demuxer {
	GSList *mediastreams;
	gint netif_nr;
	gchar *nprocess;
	gchar *nprocessfn;
	gint nprocesspid;
};


struct _mediastream {
	guchar  type;
	gushort value;
	gint demuxerpid;
	gboolean enabled;
};

struct _netgate {
	//client variables
	gint clientsock;
	gchar clientipstr[INET_ADDRSTRLEN];
	struct sockaddr_in server;
	gchar *serveraddrstr;
	guchar clienthash[32];
	gushort serveraddrport;
	gboolean clientconnected;
	//server variables
	gint serversock;
	gint serverhttpsock;
	gchar serveripstr[INET_ADDRSTRLEN];
	gchar servernodename[256];
	guchar serverhash[32];
	gushort serverport;
	gboolean serverexpsvclist;
	gboolean resetconf;
	gboolean runstatus;
	//generic
	GError *error;
	//custom
	guint32 netsvcid;
};

struct _tuneparams {
	guint   freq;
	guint   sr;
	guint   realfreq;
	guint   realsr;
	guchar  tuned;
	guchar  pol;
	guchar  fec;
	guchar  sys;
	guchar  mod;
	guchar  rof;
	guchar  port10;
	guchar  port11;
	guchar  rep11;
	guchar  lnbtype;
	guchar  lnb22khz;
};

struct _sigstatus {
	gchar svcname[256*4];
	uint16_t snr;
	uint16_t agc;
	uint32_t ber;
	gboolean lock;
	gboolean carrier;
};


struct _globalsettings {
	gboolean nonetwork;
	gboolean nostreaming;
};

typedef struct _tuneparams *tuneparams_t;
typedef struct _netgate *netgate_t;
typedef struct _streamconfig *streamconfig_t;
typedef struct _globalsettings *globalsettings_t;
typedef struct _frontend *frontend_t;
typedef struct _service *service_t;

gboolean mainwindow_tune_start(guint32 id, guchar type);
gboolean mainwindow_network_start(gchar *address, gushort port, gchar *password);
void mainwindow_network_servicelist_refresh(void);
void mainwindow_satscan_start(gboolean resume);
tuneparams_t mainwindow_get_tuneparams(void);
netgate_t mainwindow_get_netgate(void);
streamconfig_t mainwindow_get_streamconfig(void);
globalsettings_t mainwindow_get_globalsettings(void);
frontend_t mainwindow_get_frontend(void);
service_t mainwindow_get_service(void);

void mainwindow_graph_set_colors(GdkColor *mincolor, GdkColor *maxcolor);
void mainwindow_get_signal_status(struct _sigstatus *sigstatus);
gchar *mainwindow_string_definition(guint table, guint id);
gchar *mainwindow_get_client_net_address(void);
//GSList *mainwindow_get_devices(struct _frontend *frontend);
GSList *mainwindow_get_devices(gboolean rescan);
//gint mainwindow_device_search(struct _frontend *frontend);
gint mainwindow_device_search(guint adapter, guint frontend, gboolean rescan, gboolean restart);
//void mainwindow_device_open(guint devicenum, gboolean restart);


#endif /* __MAINWINDOW_H__ */
