/*
 *      scanwindow.c
 *      
 *      Copyright 2009 - 2010 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <string.h>
#include "dvbscan.h"
#include "resources.h"
#include "dbstorage.h"
#include "mainwindow.h"
#include "common/iso3166.h"

enum {
	SVC_SELECTED = 0,
	SVC_TYPE,
	SVC_NAME,
	SVC_PROVIDER,
	SVC_SID,
	SVC_PMT,
	SVC_VPID,
	SVC_APID,
	SVC_DPID,
	SVC_CAID,
	SVC_ID,
	SVC_COLUMNS
};

struct sscanwindow {
	//transponder scan window
	gboolean visible;
	GtkWidget *win;
	gchar *title;
	GtkWidget *caption;
	GtkWidget *scan;
	GtkWidget *close;
	GtkWidget *add;
	GtkWidget *list;
	GtkWidget *netname;
	GtkWidget *utctime;
	GtkWidget *progress;
	//GtkTreeSelection *selection;
	GThread *scanthread;
	guint timeout;
	gboolean threadrunning;
};
struct sscanwindow scanwindow;

struct _srcdevparams {
	guint adapter;
	guint demux;
};

struct _srcdevparams srcdevparams;


gint scanwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data);
void scanwindow_init_list(GtkWidget *list);


gint scanwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	scanwindow.visible = FALSE;
	if (!scanwindow.threadrunning) {
		dvbscan_free_list();
		if (scanwindow.timeout > 0) {
			g_source_remove(scanwindow.timeout);
			scanwindow.timeout = 0;
		}
	}
	gtk_widget_destroy(scanwindow.win);
    return FALSE;
}

void scanwindow_list_toggle(GtkCellRendererToggle *renderer, gchar *path, gpointer data)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	gint id;
	gboolean addflag = FALSE;
	struct _srcservice *scansvc;
	GHashTable *scansvchash;
	GHashTableIter hashiter;
	gpointer key, value;
	
		
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(scanwindow.list));
	if (gtk_tree_model_get_iter_from_string(model, &iter, path)) {
		gtk_tree_model_get(model, &iter, SVC_ID, &id, -1);
		scansvchash = dvbscan_get_services();
		if ((scansvc = g_hash_table_lookup(scansvchash, &id)) != NULL) {
			scansvc->sticky = !scansvc->sticky;
			gtk_list_store_set(GTK_LIST_STORE(model), &iter, SVC_SELECTED, scansvc->sticky, -1);
			
			g_hash_table_iter_init(&hashiter, scansvchash);
			while (g_hash_table_iter_next(&hashiter, &key, &value)) {
				scansvc = (struct _srcservice *)value;
				addflag = addflag || scansvc->sticky;
			}
			
			if (addflag) {
				gtk_widget_set_sensitive(scanwindow.add, TRUE);
			} else {
				gtk_widget_set_sensitive(scanwindow.add, FALSE);
			}
			
		}
	}
}

void scanwindow_list_textedit(GtkCellRendererText *renderer, gchar *path, gchar *new_text, gpointer data)
{
	GtkTreeIter iter;
	GtkTreeModel *model;
	gint option;
	struct _srcservice *scansvc;
	gint id;
	GHashTable *scansvchash;
	
	if (g_ascii_strcasecmp(new_text, "") != 0) {
		model = gtk_tree_view_get_model(GTK_TREE_VIEW(scanwindow.list));
		option = GPOINTER_TO_INT(data);
		if (gtk_tree_model_get_iter_from_string(model, &iter, path)) {
			gtk_tree_model_get(model, &iter, SVC_ID, &id, -1);
			scansvchash = dvbscan_get_services();
			if ((scansvc = g_hash_table_lookup(scansvchash, &id)) != NULL) {
				switch (option) {
					case 0: {
						g_free(scansvc->name);
						scansvc->name = g_strdup(new_text);
						gtk_list_store_set(GTK_LIST_STORE(model), &iter, SVC_NAME, new_text, -1);
						break;
					}
					case 1: {
						g_free(scansvc->provider);
						scansvc->provider = g_strdup(new_text);
						gtk_list_store_set(GTK_LIST_STORE(model), &iter, SVC_PROVIDER, new_text, -1);
						break;
					}
					default: break;
				}
			}
		}
	}
}


void scanwindow_init_list(GtkWidget *list)
{
	GtkCellRenderer    *renderer;
	GtkTreeViewColumn  *column1, *column2, *column3, *column4, *column5, *column6, *column7, *column8, *column9, *column10, *column11;
	GtkListStore       *store;
	GtkTreeIter        iter;
	/*GtkWidget          *image;
	GdkPixbuf          *pixbuf;*/
	
	renderer = gtk_cell_renderer_toggle_new();
	g_object_set(G_OBJECT(renderer), "activatable", TRUE, NULL);
	g_signal_connect(G_OBJECT(renderer), "toggled", G_CALLBACK(scanwindow_list_toggle), NULL);
	column1 = gtk_tree_view_column_new_with_attributes("", renderer, "active", SVC_SELECTED, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column1);
	column2 = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(column2, "");
	renderer = gtk_cell_renderer_pixbuf_new();
	gtk_tree_view_column_pack_start(column2, renderer, FALSE);
	gtk_tree_view_column_set_attributes(column2, renderer, "pixbuf", SVC_TYPE, NULL);
	//column2 = gtk_tree_view_column_new_with_attributes("", renderer, "pixbuf", SVC_TYPE, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column2);
	renderer = gtk_cell_renderer_text_new();
	g_object_set(renderer, "editable", TRUE, "editable-set", TRUE, NULL);
	g_signal_connect(G_OBJECT(renderer), "edited", G_CALLBACK(scanwindow_list_textedit), GINT_TO_POINTER(0));
	column3 = gtk_tree_view_column_new_with_attributes(_("Name"), renderer, "text", SVC_NAME, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column3);
	renderer = gtk_cell_renderer_text_new();
	g_object_set(renderer, "editable", TRUE, "editable-set", TRUE, NULL);
	g_signal_connect(G_OBJECT(renderer), "edited", G_CALLBACK(scanwindow_list_textedit), GINT_TO_POINTER(1));
	column4 = gtk_tree_view_column_new_with_attributes(_("Provider"), renderer, "text", SVC_PROVIDER, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column4);
	renderer = gtk_cell_renderer_text_new();
	column5 = gtk_tree_view_column_new_with_attributes(_("SID"), renderer, "text", SVC_SID, NULL);  
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column5);
	renderer = gtk_cell_renderer_text_new();
	column6 = gtk_tree_view_column_new_with_attributes(_("PMT"), renderer, "text", SVC_PMT, NULL);  
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column6);
	renderer = gtk_cell_renderer_text_new();
	column7 = gtk_tree_view_column_new_with_attributes(_("VPID"), renderer, "text", SVC_VPID, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column7);
	renderer = gtk_cell_renderer_text_new();
	column8 = gtk_tree_view_column_new_with_attributes(_("APID"), renderer, "markup", SVC_APID, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column8);
	renderer = gtk_cell_renderer_text_new();
	column9 = gtk_tree_view_column_new_with_attributes(_("DPID"), renderer, "markup", SVC_DPID, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column9);
	renderer = gtk_cell_renderer_text_new();
	column10 = gtk_tree_view_column_new_with_attributes(_("CAID"), renderer, "markup", SVC_CAID, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column10);
	
	store = gtk_list_store_new(SVC_COLUMNS, G_TYPE_BOOLEAN, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, 
											G_TYPE_INT, G_TYPE_INT, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING,
											G_TYPE_STRING, G_TYPE_INT);
	
	gtk_tree_view_set_model(GTK_TREE_VIEW(list), GTK_TREE_MODEL(store));
	g_object_unref(store);
}

gboolean scanwindow_update_progress(gpointer data)
{
	if (scanwindow.visible) {
		if (scanwindow.threadrunning) {
			gtk_progress_bar_pulse(GTK_PROGRESS_BAR(scanwindow.progress));
		} else {
			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(scanwindow.progress), 0.0);
		}
	}
	
	return scanwindow.threadrunning;
}

gchar **scanwindow_markup_string(struct _srcservice *service)
{
	gint i = 1;
	gchar **strv;
	gchar *res1, *res2, *res3;
	gboolean neednl = TRUE;
	gchar a3lang[4];
	
	
	strv = (gchar **)g_malloc(19*sizeof(gchar *));
	strv[0] = g_strdup("<small>");
	while ((service->apid[i-1] != 0) && (i-1 < 16)) {
		if ((i-1==16) || ((i-1<16) && (service->apid[i] == 0))) neednl = FALSE;
		if (service->lang[i-1] != 0) {
			strv[i] = g_strdup_printf(neednl ? "%i (%s)\n" : "%i (%s)", service->apid[i-1], iso3166_get_a3_from_code_static(service->lang[i-1], (char *)&a3lang), NULL);
		} else {
			strv[i] = g_strdup_printf(neednl ? "%i\n" : "%i", service->apid[i-1], NULL);
		}
		i++;
	}
	if (i == 1) {
		strv[i] = g_strdup("--");
		i++;
	}
	strv[i] = g_strdup("</small>");
	strv[i+1] = NULL;
	
	res1 = g_strjoinv("", strv);
	g_strfreev(strv);
	
	i = 1;
	neednl = TRUE;
	strv = (gchar **)g_malloc(19*sizeof(gchar *));
	strv[0] = g_strdup("<small>");
	while ((service->dpid[i-1] != 0) && (i-1 < 16)) {
		if ((i-1==16) || ((i-1<16) && (service->dpid[i] == 0))) neednl = FALSE;
		strv[i] = g_strdup_printf(neednl ? "%i\n" : "%i", service->dpid[i-1], NULL);
		i++;
	}
	if (i == 1) {
		strv[i] = g_strdup("--");
		i++;
	}
	strv[i] = g_strdup("</small>");
	strv[i+1] = NULL;
	
	res2 = g_strjoinv("", strv);
	g_strfreev(strv);
	
	i = 1;
	neednl = TRUE;
	strv = (gchar **)g_malloc(19*sizeof(gchar *));
	strv[0] = g_strdup("<small>");
	while ((service->caid[i-1] != 0) && (i-1 < 16)) {
		if ((i-1==16) || ((i-1<16) && (service->caid[i] == 0))) neednl = FALSE;
		gchar *tmp = dvbscan_get_cas_name(service->caid[i-1]);
		strv[i] = g_strdup_printf(neednl ? "%s\n" : "%s", tmp, NULL);
		g_free(tmp);
		i++;
	}
	if (i == 1) {
		strv[i] = g_strdup("FTA");
		i++;
	}
	strv[i] = g_strdup("</small>");
	strv[i+1] = NULL;
	
	res3 = g_strjoinv("", strv);
	g_strfreev(strv);
	
	strv = (gchar **)g_malloc(4*sizeof(gchar *));
	strv[0] = res1;
	strv[1] = res2;
	strv[2] = res3;
	strv[3] = NULL;
	
	return strv;
}

gboolean scanwindow_thread_finished(gpointer data)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	struct _srcservice *scansvc;
	gchar **markup;
	GdkPixbuf *ptv, *pradio, *pdata, *pother, *pixbuf;
	GHashTable *scansvchash;
	GHashTableIter hashiter;
	gpointer key, value;
	gchar *networkname;
	struct tm *timestamp;
	gchar timebuf[256];
	guint ftasvcs = 0;
		
	if (scanwindow.visible) {
		scansvchash = dvbscan_get_services();
		if ((scansvchash != NULL) && (g_hash_table_size(scansvchash) > 0)) {
			model = gtk_tree_view_get_model(GTK_TREE_VIEW(scanwindow.list));
			ptv = gdk_pixbuf_new_from_file(RESOURCE_SCANWINDOW_TV, NULL);
			pradio = gdk_pixbuf_new_from_file(RESOURCE_SCANWINDOW_RADIO, NULL);
			pdata = gdk_pixbuf_new_from_file(RESOURCE_SCANWINDOW_DATA, NULL);
			pother = gdk_pixbuf_new_from_file(RESOURCE_SCANWINDOW_OTHER, NULL);
			g_hash_table_iter_init(&hashiter, scansvchash);
			while (g_hash_table_iter_next(&hashiter, &key, &value)) {
				scansvc = (struct _srcservice *)value;
				markup = scanwindow_markup_string(scansvc);
				switch (scansvc->type) {
					case 0x01: pixbuf = ptv; break;
					case 0x02: pixbuf = pradio; break;
					case 0x0c: pixbuf = pdata; break;
					default:   pixbuf = pother; break;
				}
				if (scansvc->sticky) ftasvcs++;
				gtk_list_store_append(GTK_LIST_STORE(model), &iter);
				gtk_list_store_set(GTK_LIST_STORE(model), &iter, SVC_SELECTED, scansvc->sticky, SVC_TYPE, pixbuf, SVC_NAME, scansvc->name, SVC_PROVIDER, scansvc->provider,
									SVC_SID, scansvc->sid, SVC_PMT, scansvc->pmt, SVC_VPID, scansvc->vpid, SVC_APID, markup[0], SVC_DPID, markup[1],
									SVC_CAID, markup[2], SVC_ID, scansvc->sid, -1);
			}
			
			gdk_pixbuf_unref(ptv);
			gdk_pixbuf_unref(pradio);
			gdk_pixbuf_unref(pdata);
			gdk_pixbuf_unref(pother);
			g_strfreev(markup);
			
			networkname = dvbscan_get_network_name();
			if (networkname != NULL) {
				gtk_label_set_label(GTK_LABEL(scanwindow.netname), networkname);
			} else {
				gtk_label_set_label(GTK_LABEL(scanwindow.netname), "--");
			}
			
			timestamp = dvbscan_get_tptime();
			if ((timestamp != NULL) && (strftime(timebuf, sizeof(timebuf), "%c", timestamp) != 0)) {
				gtk_label_set_label(GTK_LABEL(scanwindow.utctime), timebuf);
			} else {
				gtk_label_set_label(GTK_LABEL(scanwindow.utctime), "--");
			}
						
			if (ftasvcs > 0) gtk_widget_set_sensitive(scanwindow.add, TRUE);
		}
		scanwindow.threadrunning = FALSE;
		gtk_widget_set_sensitive(scanwindow.scan, TRUE);
		gtk_widget_set_sensitive(scanwindow.close, TRUE);
	}
	
	return FALSE;
}

gboolean scanwindow_set_status(gpointer data)
{
	gint status;
	gchar statusstr[16];
	gint sl;
	
	status = GPOINTER_TO_INT(data);
	
	switch(status) {
		case DVBSCAN_STATUS_PAT:
			sl = sprintf((gchar *)&statusstr, "PAT");
			break;
		case DVBSCAN_STATUS_PMT:
			sl = sprintf((gchar *)&statusstr, "PMT");
			break;
		case DVBSCAN_STATUS_SDT:
			sl = sprintf((gchar *)&statusstr, "SDT");
			break;
		case DVBSCAN_STATUS_NIT:
			sl = sprintf((gchar *)&statusstr, "NIT");
			break;
		case DVBSCAN_STATUS_TDT:
			sl = sprintf((gchar *)&statusstr, "TDT");
			break;
		case DVBSCAN_STATUS_NULL:
			sl = sprintf((gchar *)&statusstr, " ");
			break;
		default:
			sl = sprintf((gchar *)&statusstr, " ");
			break;
	}
	
	statusstr[sl] = '\0';
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(scanwindow.progress), (gchar *)&statusstr);
	
	return FALSE;
}

void scanwindow_status_callback(gint status)
{
	g_idle_add(scanwindow_set_status, GINT_TO_POINTER(status));
}

gpointer scanwindow_thread(gpointer data)
{
	dvbscan_start(srcdevparams.adapter, srcdevparams.demux, scanwindow_status_callback);
	g_idle_add(scanwindow_thread_finished, NULL);
	g_idle_add(scanwindow_set_status, GINT_TO_POINTER(DVBSCAN_STATUS_NULL));
}

void scanwindow_scan(GtkWidget *widget, gpointer data)
{
	GtkTreeModel *model;
	
	gtk_widget_set_sensitive(scanwindow.scan, FALSE);	
	gtk_widget_set_sensitive(scanwindow.close, FALSE);
	gtk_widget_set_sensitive(scanwindow.add, FALSE);
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(scanwindow.list));
	gtk_list_store_clear(GTK_LIST_STORE(model));
	gtk_label_set_label(GTK_LABEL(scanwindow.netname), " ");
	gtk_label_set_label(GTK_LABEL(scanwindow.utctime), " ");
	scanwindow.threadrunning = TRUE;
	scanwindow.timeout = g_timeout_add(100, scanwindow_update_progress, NULL);
	scanwindow.scanthread = g_thread_create(scanwindow_thread, NULL, FALSE, NULL);
	
}

void scanwindow_add(GtkWidget *widget, gpointer data)
{
	struct _srcservice *scansvc;
	GHashTable *scansvchash;
	GHashTableIter hashiter;
	gpointer key, value;
	struct _service *svc;
	struct _tuneparams *tuneparams;
	struct _pid *curpid;
	guint i;
	GSList *svclist;
	guint svcnumber;
	guint firstapid = 0;
	gushort llcode;
	
	if ((tuneparams = mainwindow_get_tuneparams()) == NULL) return;
	
	llcode = iso3166_get_code_from_locale_env();
	
	scansvchash = dvbscan_get_services();
	svclist = NULL;
	g_hash_table_iter_init(&hashiter, scansvchash);
	while (g_hash_table_iter_next(&hashiter, &key, &value)) {
		scansvc = (struct _srcservice *)value;
		if (scansvc->sticky) {
			svc = g_malloc0(sizeof(struct _service));
			svc->id = 0;
			svc->picture = NULL;
			svc->name = g_strdup(scansvc->name);
			svc->freq = tuneparams->freq;
			svc->sr = tuneparams->sr;
			svc->pol = tuneparams->pol;
			svc->fec = tuneparams->fec;
			svc->sys = tuneparams->sys;
			svc->mod = tuneparams->mod;
			svc->rof = tuneparams->rof;
			svc->port10 = tuneparams->port10;
			svc->port11 = tuneparams->port11;
			svc->rep11 = tuneparams->rep11;
			svc->lnb22khz = tuneparams->lnb22khz;
			svc->lnbtype = tuneparams->lnbtype;
			svc->caid = scansvc->caid[0];
			svc->key = NULL;
			switch (scansvc->type) {
				case 0x01 ... 0x02: {
					if (scansvc->type == 0x01) svc->type = SERVICE_TV_RADIO; else svc->type = SERVICE_RADIO;
					svc->nip = NULL;
					svc->nmask = NULL;
					svc->nmac = NULL;
					svc->nprog = NULL;
					svc->npid = 0;
					svc->sid = scansvc->sid;
					
					if (scansvc->vpid != 0) {
						curpid = (pid *)g_malloc0(sizeof(pid));
						curpid->type = PID_TYPE_VIDEO;
						curpid->value = scansvc->vpid;
						curpid->enabled = TRUE;
						svc->pids = g_slist_prepend(svc->pids, curpid);
					}
					
					if (llcode != 0) {
						for (i=0; i<16; i++) {
							if (scansvc->apid[i] != 0) {
								if (scansvc->lang[i] == llcode) {
									firstapid = i;
									break;
								}
							} else break;
						}
					}
					
					for (i=0; i<16; i++) {
						if (scansvc->apid[i] != 0) {
							curpid = (pid *)g_malloc0(sizeof(pid));
							curpid->type = PID_TYPE_AUDIO;
							curpid->value = scansvc->apid[i];
							if (i == firstapid) {
								curpid->enabled = TRUE;
							} else {
								curpid->enabled = FALSE;
							}
							svc->pids = g_slist_prepend(svc->pids, curpid);
						} else break;
					}
					
					svc->pids = g_slist_reverse(svc->pids);
					
					break;
				}
				
				case 0x0c: {
					svc->type = SERVICE_DATA;
					svc->nip = g_strdup("192.168.0.1");
					svc->nmask = g_strdup("255.255.255.0");
					svc->nmac = g_strdup("00:01:02:03:04:05");
					svc->nprog = NULL;
					svc->npid = scansvc->dpid[0];
					svc->sid = scansvc->sid;
					svc->pids = NULL;
					break;
				}
				
				default: {
					svc->type = SERVICE_TRANSPONDER;
					svc->nip = NULL;
					svc->nmask = NULL;
					svc->nmac = NULL;
					svc->nprog = NULL;
					svc->npid = 0;
					svc->sid = 0;
					svc->pids = NULL;
					break;
				}
			}
			
			printf("[+] Added service after scan %s %u\n", svc->name, svc->freq);
			svclist = g_slist_prepend(svclist, svc);
			
		
		}
	}
	
	svclist = g_slist_reverse(svclist);
	
	if ((svcnumber = g_slist_length(svclist)) > 0) {
		printf("Transponder scan services to save: %u\n", svcnumber);
		service_add_multiple(&svclist);
		g_slist_free(svclist);
		svclist = NULL;
	}
	
	scanwindow_close(widget, NULL, data);
}

gboolean scanwindow_create(guint adapter, guint demux)
{
    GtkWidget *vbox, *hbox;
	GtkWidget *align;
	PangoFontDescription *fds;
	GdkColor color;
	GtkWidget *evbox;
	GtkWidget *chbox;
	GtkWidget *icon;
	GtkWidget *sw;
	GtkWidget *netname_l, *utctime_l;
	
	
	if (!scanwindow.visible) {
	  scanwindow.visible = TRUE;
	  srcdevparams.adapter = adapter;
	  srcdevparams.demux = demux;  
	  	  
	  scanwindow.win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	  scanwindow.title = _("Transponder structure");
	  gtk_window_set_position(GTK_WINDOW(scanwindow.win), GTK_WIN_POS_CENTER);
	  gtk_window_set_title(GTK_WINDOW(scanwindow.win), scanwindow.title);
	  //gtk_widget_set_size_request(scanwindow.win, 570, 250);
	  gtk_window_set_default_size(GTK_WINDOW(scanwindow.win), 570, 300);
	  
	  vbox = gtk_vbox_new(FALSE, 0);
	  
	  fds = pango_font_description_from_string("14");
	  pango_font_description_set_weight(fds,PANGO_WEIGHT_BOLD);
	  scanwindow.caption = gtk_label_new(_("Transponder structure"));
	  gtk_widget_modify_font(scanwindow.caption, fds);
	  pango_font_description_free(fds);
	  
	  evbox = gtk_event_box_new();
	  gtk_event_box_set_visible_window(GTK_EVENT_BOX(evbox), TRUE);
	  gdk_color_parse("white", &color);
	  gtk_widget_modify_bg(evbox, GTK_STATE_NORMAL, &color);
	  chbox = gtk_hbox_new(FALSE, 0);
	  gtk_container_set_border_width(GTK_CONTAINER(chbox), 5);
	  icon = gtk_image_new_from_stock(GTK_STOCK_FIND, GTK_ICON_SIZE_DIALOG);
	  gtk_box_pack_start(GTK_BOX(chbox), icon, FALSE, FALSE, 15);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), scanwindow.caption);
	  gtk_box_pack_start(GTK_BOX(chbox), align, FALSE, FALSE, 0);
	  gtk_container_add(GTK_CONTAINER(evbox), chbox);
	  
	  gtk_box_pack_start(GTK_BOX(vbox),evbox,FALSE,FALSE,0);
	  
	  sw = gtk_scrolled_window_new(NULL, NULL);
      gtk_container_set_border_width(GTK_CONTAINER(sw), 5);
	  scanwindow.list = gtk_tree_view_new();
	  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_ETCHED_IN);
										   
	  gtk_container_add(GTK_CONTAINER (sw), scanwindow.list);
	  //scanwindow.selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(scanwindow.list));
      scanwindow_init_list(scanwindow.list);
	  gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(scanwindow.list), TRUE);
	  gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
	  
	  hbox = gtk_hbox_new(FALSE, 0);
	  netname_l = gtk_label_new("");
	  gtk_label_set_markup(GTK_LABEL(netname_l), _("<b>Network name: </b>"));
	  gtk_box_pack_start(GTK_BOX(hbox), netname_l, FALSE, TRUE, 5);
	  
	  scanwindow.netname = gtk_label_new("");
	  gtk_label_set_max_width_chars(GTK_LABEL(scanwindow.netname), 30);
	  gtk_label_set_selectable(GTK_LABEL(scanwindow.netname), TRUE);
	  gtk_label_set_ellipsize(GTK_LABEL(scanwindow.netname), PANGO_ELLIPSIZE_END);
	  gtk_box_pack_start(GTK_BOX(hbox), scanwindow.netname, TRUE, TRUE, 5);
	  
	  utctime_l = gtk_label_new("");
	  gtk_label_set_markup(GTK_LABEL(utctime_l), _("<b>UTC time: </b>"));
	  gtk_box_pack_start(GTK_BOX(hbox), utctime_l, FALSE, TRUE, 5);
	  
	  scanwindow.utctime = gtk_label_new("");
	  gtk_label_set_max_width_chars(GTK_LABEL(scanwindow.utctime), 30);
	  gtk_label_set_selectable(GTK_LABEL(scanwindow.utctime), TRUE);
	  gtk_label_set_ellipsize(GTK_LABEL(scanwindow.utctime), PANGO_ELLIPSIZE_END);
	  gtk_box_pack_start(GTK_BOX(hbox), scanwindow.utctime, TRUE, TRUE, 5);
	  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
	  
	  scanwindow.add = gtk_button_new_from_stock(GTK_STOCK_ADD);
	  scanwindow.scan = gtk_button_new_with_label(_("Start scan"));
	  scanwindow.close = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
	  scanwindow.progress = gtk_progress_bar_new();
	  gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(scanwindow.progress), 0.0);
	  #if GTK_CHECK_VERSION(3, 0, 0)
		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(scanwindow.progress), " ");
		gtk_progress_bar_set_show_text(GTK_PROGRESS_BAR(scanwindow.progress), TRUE);
	  #endif
	 
	  hbox = gtk_hbox_new(FALSE, 0);
	  gtk_box_pack_start(GTK_BOX(hbox), scanwindow.progress, FALSE, FALSE, 5);
	  gtk_box_pack_end(GTK_BOX(hbox), scanwindow.scan, FALSE, FALSE, 5);
	  gtk_box_pack_end(GTK_BOX(hbox), scanwindow.close, FALSE, FALSE, 5);
	  gtk_box_pack_end(GTK_BOX(hbox), scanwindow.add, FALSE, FALSE, 5);
	   
	  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);

      g_signal_connect(G_OBJECT(scanwindow.win), "destroy", G_CALLBACK(scanwindow_close), NULL);
	  g_signal_connect(G_OBJECT(scanwindow.close), "clicked", G_CALLBACK(scanwindow_close), NULL);
	  g_signal_connect(G_OBJECT(scanwindow.scan), "clicked", G_CALLBACK(scanwindow_scan), NULL);
	  g_signal_connect(G_OBJECT(scanwindow.add), "clicked", G_CALLBACK(scanwindow_add), NULL);
	  
	  gtk_container_add(GTK_CONTAINER(scanwindow.win),vbox);
	  gtk_widget_show_all(scanwindow.win);
	  
	  gtk_window_set_focus(GTK_WINDOW(scanwindow.win), scanwindow.scan);
	  gtk_widget_set_sensitive(scanwindow.add, FALSE);
	  
	  if (scanwindow.threadrunning) {
		  gtk_widget_set_sensitive(scanwindow.scan, FALSE);
	  }
  
  } else {
	  
	  gtk_window_present(GTK_WINDOW(scanwindow.win));
	  
  } 
  
      return TRUE;
}
