/*
 *      strcliwindow.c
 *      
 *      Copyright 2009 - 2010 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <string.h>

#include "mainwindow.h"
#include "streamconfig.h"

enum {
	STRCLI_COL_ICON = 0,
	STRCLI_COL_PID,
	STRCLI_COL_ID,
	STRCLI_COL_TIME,
	STRCLI_COL_ADDR,
	STRCLI_COL_USERAGENT,
	STRCLI_COL_STOPABLE,
	STRCLI_COLUMNS
};

struct sstrcliwindow {
	//streamserver clients window
	gboolean visible;
	GtkWidget *win;
	gchar *title;
	GtkWidget *caption;
	GtkWidget *close;
	GtkWidget *stop;
	GtkWidget *list;
	GtkTreeSelection *selection;
	guint timer;
};
struct sstrcliwindow strcliwindow;


gint strcliwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data);
void strcliwindow_init_list(GtkWidget *list);


gint strcliwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	strcliwindow.visible = FALSE;
	g_source_remove(strcliwindow.timer);
	gtk_widget_destroy(strcliwindow.win);
	return FALSE;
}

void strcliwindow_init_list(GtkWidget *list)
{
	GtkCellRenderer    *renderer;
	GtkTreeViewColumn  *column1, *column2, *column3, *column4, *column5, *column6;
	GtkListStore       *store;
	GtkTreeIter        iter;
	
		
	renderer = gtk_cell_renderer_pixbuf_new();
	column1 = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(column1, "");
	gtk_tree_view_column_pack_start(column1, renderer, FALSE);
	gtk_tree_view_column_set_attributes(column1, renderer, "stock-id", STRCLI_COL_ICON, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column1);
	
	renderer = gtk_cell_renderer_text_new();
	column2 = gtk_tree_view_column_new_with_attributes(_("PID"), renderer, "markup", STRCLI_COL_PID, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column2);
	
	renderer = gtk_cell_renderer_text_new();
	column3 = gtk_tree_view_column_new_with_attributes(_("ID"), renderer, "markup", STRCLI_COL_ID, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column3);
	
	renderer = gtk_cell_renderer_text_new();
	column4 = gtk_tree_view_column_new_with_attributes(_("Time"), renderer, "markup", STRCLI_COL_TIME, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column4);
	
	renderer = gtk_cell_renderer_text_new();
	column5 = gtk_tree_view_column_new_with_attributes(_("Address"), renderer, "markup", STRCLI_COL_ADDR, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column5);
	
	renderer = gtk_cell_renderer_text_new();
	g_object_set(renderer, "ellipsize", PANGO_ELLIPSIZE_END, "ellipsize-set", TRUE, NULL);
	column6 = gtk_tree_view_column_new_with_attributes(_("User agent"), renderer, "markup", STRCLI_COL_USERAGENT, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column6);
		
	store = gtk_list_store_new(STRCLI_COLUMNS, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
	
	gtk_tree_view_set_model(GTK_TREE_VIEW(list), GTK_TREE_MODEL(store));
	g_object_unref(store);
}

void strcliwindow_edit_client(struct _strclient *strclient, GtkTreeModel *model, GtkTreeIter *iter)
{
	struct tm *tms;
	gchar tmstr[30];
	gchar *iconstr;
	gboolean stopable;
	
	if ((strclient != NULL) && (model != NULL) && (iter != NULL)) {
		switch (strclient->family) {
			case CLIENT_FAMILY_HELPER:
				iconstr = GTK_STOCK_EXECUTE;
				break;
			case CLIENT_FAMILY_V4_TRANSPORT:
				iconstr = GTK_STOCK_CONNECT;
				break;
			case CLIENT_FAMILY_V6_TRANSPORT:
				iconstr = GTK_STOCK_CONNECT;
				break;
			case CLIENT_FAMILY_STOPPED:
				iconstr = GTK_STOCK_DELETE;
				break;
			default:
				iconstr = GTK_STOCK_EXECUTE;
				break;
		}
		
		if (strclient->type == CLIENT_TYPE_TCP_UNICAST) {
			stopable = TRUE;
		} else {
			stopable = FALSE;
		}
		
		memset((char *)&tmstr, 0, sizeof(tmstr));
		
		if ((tms = localtime((time_t *)&strclient->starttime)) != NULL) {
			strftime(tmstr, sizeof(tmstr), "%T", tms);
		} else {
			sprintf((char *)&tmstr, "Unknown");
		}
		
		gtk_list_store_set(GTK_LIST_STORE(model), iter, STRCLI_COL_ICON, iconstr, STRCLI_COL_PID, strclient->pid, STRCLI_COL_ID, strclient->seq, STRCLI_COL_TIME, tmstr, STRCLI_COL_ADDR, strclient->address, STRCLI_COL_USERAGENT, strclient->useragent, STRCLI_COL_STOPABLE, stopable, -1);
	}
}

void strcliwindow_add_client_foreach(gpointer data, gpointer user_data)
{
	struct _strclient *strclient;
	struct tm *tms;
	gchar tmstr[30];
	GtkTreeIter iter;
	gchar *iconstr;
	gboolean stopable;
	GtkTreeModel *model;
	
	strclient = (struct _strclient *)data;
	model = (GtkTreeModel *)user_data;
	
	if ((!strclient->sticky) && (model != NULL)) {
		switch (strclient->family) {
			case CLIENT_FAMILY_HELPER:
				iconstr = GTK_STOCK_EXECUTE;
				break;
			case CLIENT_FAMILY_V4_TRANSPORT:
				iconstr = GTK_STOCK_CONNECT;
				break;
			case CLIENT_FAMILY_V6_TRANSPORT:
				iconstr = GTK_STOCK_CONNECT;
				break;
			case CLIENT_FAMILY_STOPPED:
				iconstr = GTK_STOCK_DELETE;
				break;
			default:
				iconstr = GTK_STOCK_EXECUTE;
				break;
		}
		
		if (strclient->type == CLIENT_TYPE_TCP_UNICAST) {
			stopable = TRUE;
		} else {
			stopable = FALSE;
		}
		
		memset((char *)&tmstr, 0, sizeof(tmstr));
		
		if ((tms = localtime((time_t *)&strclient->starttime)) != NULL) {
			strftime(tmstr, sizeof(tmstr), "%T", tms);
		} else {
			sprintf((char *)&tmstr, "Unknown");
		}
		
		gtk_list_store_append(GTK_LIST_STORE(model), &iter);
		gtk_list_store_set(GTK_LIST_STORE(model), &iter, STRCLI_COL_ICON, iconstr, STRCLI_COL_PID, strclient->pid, STRCLI_COL_ID, strclient->seq, STRCLI_COL_TIME, tmstr, STRCLI_COL_ADDR, strclient->address, STRCLI_COL_USERAGENT, strclient->useragent, STRCLI_COL_STOPABLE, stopable, -1);
	}
	
}


gboolean strcliwindow_update(gpointer data)
{
	GtkTreeModel *model;
	streamconfig_t streamconfig;
	GSList *clients;
	guint clinum;
	GtkTreeIter iter;
	gboolean valid;
	gint ident, i;
	struct _strclient *strclient;
	gboolean rowfound;
	GList *removelist = NULL;
	GtkTreePath *path;
	GtkTreeRowReference *reference;
	GList *node;
	
	if (!strcliwindow.visible) return FALSE;
	
	if ((streamconfig = mainwindow_get_streamconfig()) == NULL) return TRUE;
	
	if ((model = gtk_tree_view_get_model(GTK_TREE_VIEW(strcliwindow.list))) != NULL) {
		if ((clients = streamconfig_get_clients(streamconfig)) != NULL) {
			clinum = g_slist_length(clients);
			
			valid = gtk_tree_model_get_iter_first(model, &iter);
			while (valid) {
				gtk_tree_model_get(model, &iter, STRCLI_COL_ID, &ident, -1);
				
				rowfound = FALSE;
				for (i=0; i<clinum; i++) {
					strclient = (struct _strclient *)g_slist_nth_data(clients, i);
					if (strclient->seq == ident) {
						strcliwindow_edit_client(strclient, model, &iter);
						rowfound = TRUE;
						strclient->sticky = TRUE;
						break;
					}
				}
				
				if (!rowfound) {
					//remove this client from list
					path = gtk_tree_model_get_path(model, &iter);
					reference = gtk_tree_row_reference_new(model, path);
					removelist = g_list_append(removelist, reference);
					gtk_tree_path_free(path);
				}
				
				valid = gtk_tree_model_iter_next(model, &iter);
			}
			
			for (node = removelist; node != NULL; node = node->next) {
				if ((path = gtk_tree_row_reference_get_path((GtkTreeRowReference *)node->data)) != NULL) {
					if (gtk_tree_model_get_iter(GTK_TREE_MODEL(model), &iter, path)) {
						gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
					}
				}
			}
			g_list_foreach(removelist, (GFunc)gtk_tree_row_reference_free, NULL);
			g_list_free(removelist);
			
			g_slist_foreach(clients, strcliwindow_add_client_foreach, model);
			
			streamconfig_free_clients(clients);
		}
	}
	
	return TRUE;
}
	

gint strcliwindow_stop(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	GtkTreeSelection *selection;
	GtkTreeModel *model;
	GtkTreeIter iter;
	gint ident;
	gboolean stopable;
	streamconfig_t streamconfig;
	
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(strcliwindow.list));
	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get(model, &iter, STRCLI_COL_ID, &ident, STRCLI_COL_STOPABLE, &stopable, -1);
		if (stopable) {
			if ((streamconfig = mainwindow_get_streamconfig()) != NULL) {
				streamconfig_stop_client(streamconfig, ident);
			}
		} else {
			g_print("[-] Unable to stop client\n");
		}
	} else {
		g_print("[-] Unable to stop client - nothing selected\n");
	}
}

void strcliwindow_selection(GtkTreeSelection *selection, gpointer data)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	gboolean stopable;
	
	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get(model, &iter, STRCLI_COL_STOPABLE, &stopable, -1);
		gtk_widget_set_sensitive(GTK_WIDGET(strcliwindow.stop), stopable);
	} else {
		gtk_widget_set_sensitive(GTK_WIDGET(strcliwindow.stop), FALSE);
	}
	
}


gboolean strcliwindow_create(void)
{
	GtkWidget *vbox, *hbox;
	GtkWidget *align;
	PangoFontDescription *fds;
	GdkColor color;
	GtkWidget *evbox;
	GtkWidget *chbox;
	GtkWidget *icon;
	GtkWidget *sw;
	
	if (!strcliwindow.visible) {
		strcliwindow.visible = TRUE;
		
		strcliwindow.win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		strcliwindow.title = _("Stream server clients");
		gtk_window_set_position(GTK_WINDOW(strcliwindow.win), GTK_WIN_POS_CENTER);
		gtk_window_set_title(GTK_WINDOW(strcliwindow.win), strcliwindow.title);
		gtk_window_set_default_size(GTK_WINDOW(strcliwindow.win), 570, 300);
		
		vbox = gtk_vbox_new(FALSE, 0);
		
		fds = pango_font_description_from_string("14");
		pango_font_description_set_weight(fds,PANGO_WEIGHT_BOLD);
		strcliwindow.caption = gtk_label_new(_("Stream server clients"));
		gtk_widget_modify_font(strcliwindow.caption, fds);
		pango_font_description_free(fds);
		
		evbox = gtk_event_box_new();
		gtk_event_box_set_visible_window(GTK_EVENT_BOX(evbox), TRUE);
		gdk_color_parse("white", &color);
		gtk_widget_modify_bg(evbox, GTK_STATE_NORMAL, &color);
		chbox = gtk_hbox_new(FALSE, 0);
		gtk_container_set_border_width(GTK_CONTAINER(chbox), 5);
		icon = gtk_image_new_from_stock(GTK_STOCK_CONNECT, GTK_ICON_SIZE_DIALOG);
		gtk_box_pack_start(GTK_BOX(chbox), icon, FALSE, FALSE, 15);
		align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
		gtk_container_add(GTK_CONTAINER(align), strcliwindow.caption);
		gtk_box_pack_start(GTK_BOX(chbox), align, FALSE, FALSE, 0);
		gtk_container_add(GTK_CONTAINER(evbox), chbox);
		
		gtk_box_pack_start(GTK_BOX(vbox),evbox,FALSE,FALSE,0);
		
		sw = gtk_scrolled_window_new(NULL, NULL);
		gtk_container_set_border_width(GTK_CONTAINER(sw), 5);
		strcliwindow.list = gtk_tree_view_new();
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
		gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_ETCHED_IN);
		
		gtk_container_add(GTK_CONTAINER (sw), strcliwindow.list);
		strcliwindow.selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(strcliwindow.list));
		gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(strcliwindow.list)), GTK_SELECTION_BROWSE);
		
		strcliwindow_init_list(strcliwindow.list);
		
		gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(strcliwindow.list), TRUE);
		gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
		
		strcliwindow.close = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
		strcliwindow.stop = gtk_button_new_from_stock(GTK_STOCK_STOP);
		
		hbox = gtk_hbox_new(FALSE, 0);
		gtk_box_pack_end(GTK_BOX(hbox), strcliwindow.close, FALSE, FALSE, 5);
		gtk_box_pack_end(GTK_BOX(hbox), strcliwindow.stop, FALSE, FALSE, 5);
		
		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
		
		g_signal_connect(G_OBJECT(strcliwindow.win), "destroy", G_CALLBACK(strcliwindow_close), NULL);
		g_signal_connect(G_OBJECT(strcliwindow.close), "clicked", G_CALLBACK(strcliwindow_close), NULL);
		g_signal_connect(G_OBJECT(strcliwindow.stop), "clicked", G_CALLBACK(strcliwindow_stop), NULL);
		g_signal_connect(G_OBJECT(strcliwindow.selection), "changed", G_CALLBACK(strcliwindow_selection), NULL);
		
		gtk_container_add(GTK_CONTAINER(strcliwindow.win),vbox);
		gtk_widget_show_all(strcliwindow.win);
		
		gtk_widget_set_sensitive(GTK_WIDGET(strcliwindow.stop), FALSE);
		
		gtk_window_set_focus(GTK_WINDOW(strcliwindow.win), strcliwindow.list);
		
		strcliwindow_update(NULL);
		strcliwindow.timer = g_timeout_add(1000, strcliwindow_update, NULL);
	} else {
		gtk_window_present(GTK_WINDOW(strcliwindow.win));
	}
	
	return TRUE;
}
