/*
 *      tswindow.c
 *      
 *      Copyright 2009-2011 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <string.h>

#include "mainwindow.h"
#include "streamconfig.h"

enum {
	TS_COL_ICON = 0,
	TS_COL_PID,
	TS_COL_BANDWIDTH,
	TS_COL_PERCENTAGE,
	TS_COL_STREAMPARAMS,
	TS_COLUMNS
};

struct stswindow {
	//log browser window
	gboolean visible;
	GtkWidget *win;
	gchar *title;
	GtkWidget *caption;
	GtkWidget *close;
	GtkWidget *list;
	GtkTreeModel *model;
	guint timer;
};
struct stswindow tswindow;


gint tswindow_close(GtkWidget *widget, GdkEvent *event, gpointer data);
void tswindow_init_list(GtkWidget *list);
void tswindow_edit_stream(struct _strtspid *strtspid, GtkTreeModel *model, GtkTreeIter *iter);
void tswindow_add_stream_foreach(gpointer data, gpointer user_data);
gboolean tswindow_update(gpointer data);


gint tswindow_close(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	tswindow.visible = FALSE;
	g_source_remove(tswindow.timer);
	gtk_widget_destroy(tswindow.win);
	return FALSE;
}

void tswindow_init_list(GtkWidget *list)
{
	GtkCellRenderer    *renderer;
	GtkTreeViewColumn  *column1, *column2, *column3, *column4, *column5;
	GtkListStore       *store;
	GtkTreeIter        iter;
	
	
	renderer = gtk_cell_renderer_pixbuf_new();
	column1 = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(column1, "");
	gtk_tree_view_column_pack_start(column1, renderer, FALSE);
	gtk_tree_view_column_set_attributes(column1, renderer, "stock-id", TS_COL_ICON, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column1);
	
	renderer = gtk_cell_renderer_text_new();
	column2 = gtk_tree_view_column_new_with_attributes(_("PID"), renderer, "text", TS_COL_PID, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column2);
	
	renderer = gtk_cell_renderer_text_new();
	column3 = gtk_tree_view_column_new_with_attributes(_("Bandwidth"), renderer, "text", TS_COL_BANDWIDTH, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column3);
	
	renderer = gtk_cell_renderer_progress_new();
	column4 = gtk_tree_view_column_new_with_attributes(_("Percentage"), renderer, "value", TS_COL_PERCENTAGE, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column4);
	
	renderer = gtk_cell_renderer_text_new();
	g_object_set(renderer, "ellipsize", PANGO_ELLIPSIZE_END, "ellipsize-set", TRUE, NULL);
	column5 = gtk_tree_view_column_new_with_attributes(_("Stream properties"), renderer, "markup", TS_COL_STREAMPARAMS, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column5);
		
	store = gtk_list_store_new(TS_COLUMNS, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT, G_TYPE_INT, G_TYPE_STRING);
	
	gtk_tree_view_set_model(GTK_TREE_VIEW(list), GTK_TREE_MODEL(store));
	g_object_unref(store);
}

void tswindow_edit_stream(struct _strtspid *strtspid, GtkTreeModel *model, GtkTreeIter *iter)
{
	gchar *iconstr;
	
	if ((strtspid != NULL) && (model != NULL) && (iter != NULL)) {
		if (strtspid->format == TS_STREAM_FORMAT_SPECIAL) {
			iconstr = GTK_STOCK_PROPERTIES;
		} else if (strtspid->format == TS_STREAM_FORMAT_TABLE) {
			iconstr = GTK_STOCK_FILE;
		} else if (strtspid->format == TS_STREAM_FORMAT_UNKNOWN) {
			switch (strtspid->type) {
				case TS_STREAM_PES_PRIVATE_STREAM_1:
					iconstr = GTK_STOCK_PROPERTIES;
					break;
				case TS_STREAM_PES_AUDIO_STREAM:
					iconstr = GTK_STOCK_MEDIA_PLAY;
					break;
				case TS_STREAM_PES_VIDEO_STREAM:
					iconstr = GTK_STOCK_MEDIA_PLAY;
					break;
				default:
					iconstr = GTK_STOCK_PROPERTIES;
					break;
			}
		} else if (strtspid->format == TS_STREAM_FORMAT_ENCRYPTED) {
			iconstr = GTK_STOCK_DIALOG_AUTHENTICATION;
		} else if (strtspid->format == TS_STREAM_FORMAT_VIDEO_H262) {
			iconstr = GTK_STOCK_MEDIA_PLAY;
		} else if (strtspid->format == TS_STREAM_FORMAT_VIDEO_H264) {
			iconstr = GTK_STOCK_MEDIA_PLAY;
		} else if (strtspid->format == TS_STREAM_FORMAT_AUDIO_MPEG) {
			iconstr = GTK_STOCK_MEDIA_PLAY;
		} else if (strtspid->format == TS_STREAM_FORMAT_AUDIO_AC3) {
			iconstr = GTK_STOCK_MEDIA_PLAY;
		} else if (strtspid->format == TS_STREAM_FORMAT_AUDIO_AAC) {
			iconstr = GTK_STOCK_MEDIA_PLAY;
		} else if (strtspid->format == TS_STREAM_FORMAT_AUDIO_DTS) {
			iconstr = GTK_STOCK_MEDIA_PLAY;
		}
		
		gtk_list_store_set(GTK_LIST_STORE(model), iter, TS_COL_ICON, iconstr, TS_COL_PID, strtspid->pid, TS_COL_BANDWIDTH, strtspid->bandwidth, TS_COL_PERCENTAGE, strtspid->percentage, TS_COL_STREAMPARAMS, strtspid->description, -1);
	}
}

void tswindow_add_stream_foreach(gpointer data, gpointer user_data)
{
	struct _strtspid *strtspid;
	GtkTreeIter iter;
	gchar *iconstr;
	GtkTreeModel *model;
	
	strtspid = (struct _strtspid *)data;
	model = (GtkTreeModel *)user_data;
	
	if ((!strtspid->sticky) && (model != NULL)) {
		if (strtspid->format == TS_STREAM_FORMAT_SPECIAL) {
			iconstr = GTK_STOCK_PROPERTIES;
		} else if (strtspid->format == TS_STREAM_FORMAT_TABLE) {
			iconstr = GTK_STOCK_FILE;
		} else if (strtspid->format == TS_STREAM_FORMAT_UNKNOWN) {
			switch (strtspid->type) {
				case TS_STREAM_PES_PRIVATE_STREAM_1:
					iconstr = GTK_STOCK_PROPERTIES;
					break;
				case TS_STREAM_PES_AUDIO_STREAM:
					iconstr = GTK_STOCK_MEDIA_PLAY;
					break;
				case TS_STREAM_PES_VIDEO_STREAM:
					iconstr = GTK_STOCK_MEDIA_PLAY;
					break;
				default:
					iconstr = GTK_STOCK_PROPERTIES;
					break;
			}
		} else if (strtspid->format == TS_STREAM_FORMAT_ENCRYPTED) {
			iconstr = GTK_STOCK_DIALOG_AUTHENTICATION;
		} else if (strtspid->format == TS_STREAM_FORMAT_VIDEO_H262) {
			iconstr = GTK_STOCK_MEDIA_PLAY;
		} else if (strtspid->format == TS_STREAM_FORMAT_VIDEO_H264) {
			iconstr = GTK_STOCK_MEDIA_PLAY;
		} else if (strtspid->format == TS_STREAM_FORMAT_AUDIO_MPEG) {
			iconstr = GTK_STOCK_MEDIA_PLAY;
		} else if (strtspid->format == TS_STREAM_FORMAT_AUDIO_AC3) {
			iconstr = GTK_STOCK_MEDIA_PLAY;
		} else if (strtspid->format == TS_STREAM_FORMAT_AUDIO_AAC) {
			iconstr = GTK_STOCK_MEDIA_PLAY;
		} else if (strtspid->format == TS_STREAM_FORMAT_AUDIO_DTS) {
			iconstr = GTK_STOCK_MEDIA_PLAY;
		}
		
		gtk_list_store_append(GTK_LIST_STORE(model), &iter);
		gtk_list_store_set(GTK_LIST_STORE(model), &iter, TS_COL_ICON, iconstr, TS_COL_PID, strtspid->pid, TS_COL_BANDWIDTH, strtspid->bandwidth, TS_COL_PERCENTAGE, strtspid->percentage, TS_COL_STREAMPARAMS, strtspid->description, -1);
	}
	
}

gboolean tswindow_update(gpointer data)
{
	GtkTreeModel *model;
	streamconfig_t streamconfig;
	GSList *tsstruct;
	guint streamnum;
	GtkTreeIter iter;
	gboolean valid;
	gint ident, i;
	struct _strtspid *strtspid;
	gboolean rowfound;
	GList *removelist = NULL;
	GtkTreePath *path;
	GtkTreeRowReference *reference;
	GList *node;
	
	if (!tswindow.visible) return FALSE;
	
	if ((streamconfig = mainwindow_get_streamconfig()) == NULL) return TRUE;
	
	if ((model = gtk_tree_view_get_model(GTK_TREE_VIEW(tswindow.list))) != NULL) {
		if ((tsstruct = streamconfig_get_tsstruct(streamconfig)) != NULL) {
			streamnum = g_slist_length(tsstruct);
			
			valid = gtk_tree_model_get_iter_first(model, &iter);
			while (valid) {
				gtk_tree_model_get(model, &iter, TS_COL_PID, &ident, -1);
				
				rowfound = FALSE;
				for (i=0; i<streamnum; i++) {
					strtspid = (struct _strtspid *)g_slist_nth_data(tsstruct, i);
					if (strtspid->pid == ident) {
						tswindow_edit_stream(strtspid, model, &iter);
						rowfound = TRUE;
						strtspid->sticky = TRUE;
						break;
					}
				}
				
				if (!rowfound) {
					//remove this row from list
					path = gtk_tree_model_get_path(model, &iter);
					reference = gtk_tree_row_reference_new(model, path);
					removelist = g_list_append(removelist, reference);
					gtk_tree_path_free(path);
				}
				
				valid = gtk_tree_model_iter_next(model, &iter);
			}
			
			for (node = removelist; node != NULL; node = node->next) {
				if ((path = gtk_tree_row_reference_get_path((GtkTreeRowReference *)node->data)) != NULL) {
					if (gtk_tree_model_get_iter(GTK_TREE_MODEL(model), &iter, path)) {
						gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
					}
				}
			}
			g_list_foreach(removelist, (GFunc)gtk_tree_row_reference_free, NULL);
			g_list_free(removelist);
			
			g_slist_foreach(tsstruct, tswindow_add_stream_foreach, model);
			
			streamconfig_free_tsstruct(tsstruct);
		}
	}
	
	return TRUE;
}

gboolean tswindow_create(void)
{
	GtkWidget *vbox, *hbox;
	GtkWidget *align;
	PangoFontDescription *fds;
	GdkColor color;
	GtkWidget *evbox;
	GtkWidget *chbox;
	GtkWidget *icon;
	GtkWidget *sw;
	
	if (!tswindow.visible) {
		tswindow.visible = TRUE;
		
		tswindow.win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		tswindow.title = _("Transport stream structure");
		gtk_window_set_position(GTK_WINDOW(tswindow.win), GTK_WIN_POS_CENTER);
		gtk_window_set_title(GTK_WINDOW(tswindow.win), tswindow.title);
		//gtk_widget_set_size_request(tswindow.win, 570, 250);
		gtk_window_set_default_size(GTK_WINDOW(tswindow.win), 570, 300);
		
		vbox = gtk_vbox_new(FALSE, 0);
		
		fds = pango_font_description_from_string("14");
		pango_font_description_set_weight(fds,PANGO_WEIGHT_BOLD);
		tswindow.caption = gtk_label_new(_("Transport stream structure"));
		gtk_widget_modify_font(tswindow.caption, fds);
		pango_font_description_free(fds);
		
		evbox = gtk_event_box_new();
		gtk_event_box_set_visible_window(GTK_EVENT_BOX(evbox), TRUE);
		gdk_color_parse("white", &color);
		gtk_widget_modify_bg(evbox, GTK_STATE_NORMAL, &color);
		chbox = gtk_hbox_new(FALSE, 0);
		gtk_container_set_border_width(GTK_CONTAINER(chbox), 5);
		icon = gtk_image_new_from_stock(GTK_STOCK_OPEN, GTK_ICON_SIZE_DIALOG);
		gtk_box_pack_start(GTK_BOX(chbox), icon, FALSE, FALSE, 15);
		align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
		gtk_container_add(GTK_CONTAINER(align), tswindow.caption);
		gtk_box_pack_start(GTK_BOX(chbox), align, FALSE, FALSE, 0);
		gtk_container_add(GTK_CONTAINER(evbox), chbox);
		
		gtk_box_pack_start(GTK_BOX(vbox),evbox,FALSE,FALSE,0);
		
		sw = gtk_scrolled_window_new(NULL, NULL);
		gtk_container_set_border_width(GTK_CONTAINER(sw), 5);
		tswindow.list = gtk_tree_view_new();
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
		gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_ETCHED_IN);
		
		gtk_container_add(GTK_CONTAINER (sw), tswindow.list);
		//tswindow.selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tswindow.list));
		gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(tswindow.list)), GTK_SELECTION_BROWSE);
		
		tswindow_init_list(tswindow.list);
		
		gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(tswindow.list), TRUE);
		gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
		
		tswindow.close = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
		
		hbox = gtk_hbox_new(FALSE, 0);
		gtk_box_pack_end(GTK_BOX(hbox), tswindow.close, FALSE, FALSE, 5);
		
		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
		
		g_signal_connect(G_OBJECT(tswindow.win), "destroy", G_CALLBACK(tswindow_close), NULL);
		g_signal_connect(G_OBJECT(tswindow.close), "clicked", G_CALLBACK(tswindow_close), NULL);
		
		gtk_container_add(GTK_CONTAINER(tswindow.win),vbox);
		gtk_widget_show_all(tswindow.win);
		gtk_window_set_focus(GTK_WINDOW(tswindow.win), tswindow.list);
		
		tswindow_update(NULL);
		tswindow.timer = g_timeout_add(1000, tswindow_update, NULL);
	} else {
		gtk_window_present(GTK_WINDOW(tswindow.win));
	}
	
	return TRUE;
}
